@@ -2,7 +2,7 @@
 #
 # autogen.sh - Generates the initial makefiles from a pristine CVS tree
 #
-# $Id: autogen.sh,v 1.31 2005/02/04 19:53:51 cmdjb Exp $
+# $Id: autogen.sh,v 1.32 2005/04/17 14:19:26 cmdjb Exp $
 #
 # USAGE: autogen.sh [configure options]
 #
@@ -31,6 +31,7 @@ fi
 program=`basename $0`
 
 # automake 1.8 requires autoconf 2.58
+# automake 1.9 requires autoconf 2.58
 # automake 1.7 requires autoconf 2.54
 automake_min_vers=1.7
 aclocal_min_vers=$automake_min_vers
@@ -52,15 +53,15 @@ IFS=":"
 set - $PATH
 IFS="$save_ifs"
 
-autoconf=autoconf
+autoconf=${AUTOCONF-autoconf}
 autoconf_vers=0
-automake=automake
+automake=${AUTOMAKE-automake}
 automake_vers=0
-aclocal=aclocal
+aclocal=${ACLOCAL-aclocal}
 aclocal_vers=0
-autoheader=autoheader
+autoheader=${AUTOHEADER-autoheader}
 autoheader_vers=0
-libtoolize=libtoolize
+libtoolize=${LIBTOOLIZE-libtoolize}
 libtoolize_vers=0
 
 here=`pwd`
