@@ -2,7 +2,7 @@
  *
  * redland.h - Redland RDF Application Framework public API
  *
- * $Id: librdf.h,v 1.51 2005/01/30 23:15:06 cmdjb Exp $
+ * $Id: librdf.h,v 1.53 2005/05/20 15:04:50 cmdjb Exp $
  *
  * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -89,13 +89,13 @@ typedef struct librdf_serializer_s librdf_serializer;
 typedef struct librdf_serializer_factory_s librdf_serializer_factory;
 
 /* Public statics */
-extern const char * const librdf_short_copyright_string;
-extern const char * const librdf_copyright_string;
-extern const char * const librdf_version_string;
-extern const unsigned int librdf_version_major;
-extern const unsigned int librdf_version_minor;
-extern const unsigned int librdf_version_release;
-extern const unsigned int librdf_version_decimal;
+REDLAND_API extern const char * const librdf_short_copyright_string;
+REDLAND_API extern const char * const librdf_copyright_string;
+REDLAND_API extern const char * const librdf_version_string;
+REDLAND_API extern const unsigned int librdf_version_major;
+REDLAND_API extern const unsigned int librdf_version_minor;
+REDLAND_API extern const unsigned int librdf_version_release;
+REDLAND_API extern const unsigned int librdf_version_decimal;
 
 /* Required for va_list in error handler function registrations
  * which are in the public API
@@ -116,6 +116,8 @@ extern const unsigned int librdf_version_decimal;
  
 #ifndef LIBRDF_OBJC_FRAMEWORK
 #include <rdf_log.h>
+#include <rdf_digest.h>
+#include <rdf_hash.h>
 #include <rdf_init.h>
 #include <rdf_iterator.h>
 #include <rdf_uri.h>
@@ -130,6 +132,8 @@ extern const unsigned int librdf_version_decimal;
 #include <rdf_query.h>
 #else
 #include <Redland/rdf_log.h>
+#include <Redland/rdf_digest.h>
+#include <Redland/rdf_hash.h>
 #include <Redland/rdf_init.h>
 #include <Redland/rdf_iterator.h>
 #include <Redland/rdf_uri.h>
