@@ -2,9 +2,9 @@
  *
  * rdf_digest.c - RDF Digest Factory / Digest implementation
  *
- * $Id: rdf_digest.c,v 1.46 2004/11/01 20:43:32 cmdjb Exp $
+ * $Id: rdf_digest.c,v 1.50 2005/05/23 13:18:55 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -157,6 +157,14 @@ librdf_get_digest_factory(librdf_world *world, const char *name)
  * @world: redland world object
  * @name: the digest name to use to create this digest
  * 
+ * After construction, data should be added to the digest using
+ * &librdf_digest_update or &librdf_digest_update_string with
+ * &librdf_digest_final to signify finishing.  Then the digest
+ * value can be returned directly with &librdf_digest_get_digest
+ * of &librdf_digest_get_digest_length bytes or as a hex encoded
+ * string with &librdf_digest_to_string.  The digest can be
+ * re-initialised for new data with &librdf_digest_init.
+ *
  * Return value: new &librdf_digest object or NULL
  **/
 librdf_digest*
@@ -177,6 +185,14 @@ librdf_new_digest(librdf_world *world, char *name)
  * @world: redland world object
  * @factory: the digest factory to use to create this digest
  * 
+ * After construction, data should be added to the digest using
+ * &librdf_digest_update or &librdf_digest_update_string with
+ * &librdf_digest_final to signify finishing.  Then the digest
+ * value can be returned directly with &librdf_digest_get_digest
+ * of &librdf_digest_get_digest_length bytes or as a hex encoded
+ * string with &librdf_digest_to_string.  The digest can be
+ * re-initialised for new data with &librdf_digest_init.
+ *
  * Return value: new &librdf_digest object or NULL
  **/
 librdf_digest*
@@ -203,9 +219,11 @@ librdf_new_digest_from_factory(librdf_world *world,
     librdf_free_digest(d);
     return NULL;
   }
-  
+
   d->factory=factory;
   
+  d->factory->init(d->context);
+
   return d;
 }
 
@@ -233,6 +251,9 @@ librdf_free_digest(librdf_digest *digest)
  * librdf_digest_init - (Re)initialise the librdf_digest object
  * @digest: the digest
  * 
+ * This is automatically called on construction but can be used to
+ * re-initialise the digest to the initial state for digesting new
+ * data.
  **/
 void
 librdf_digest_init(librdf_digest* digest) 
@@ -249,12 +270,28 @@ librdf_digest_init(librdf_digest* digest)
  * 
  **/
 void
-librdf_digest_update(librdf_digest* digest, unsigned char *buf, size_t length) 
+librdf_digest_update(librdf_digest* digest, 
+                     const unsigned char *buf, size_t length) 
 {
   digest->factory->update(digest->context, buf, length);
 }
 
 
+/**
+ * librdf_digest_update_string - Add a string to the librdf_digest object
+ * @digest: the digest
+ * @string: string to add
+ * 
+ **/
+void
+librdf_digest_update_string(librdf_digest* digest, 
+                            const unsigned char *string)
+{
+  digest->factory->update(digest->context, string,
+                          strlen((const char*)string));
+}
+
+
 /**
  * librdf_digest_final - Finish the digesting of data
  * @digest: the digest
@@ -278,7 +315,7 @@ librdf_digest_final(librdf_digest* digest)
  * @digest: the digest
  * 
  * Return value: pointer to the memory containing the digest.  It will
- * be digest_factory->digest_length bytes in length.
+ * be &librdf_digest_get_digest_length bytes in length.
  *
  **/
 void*
@@ -288,6 +325,20 @@ librdf_digest_get_digest(librdf_digest* digest)
 }
 
 
+/**
+ * librdf_digest_get_digest_length - Get length of the calculated digested
+ * @digest: the digest
+ * 
+ * Return value: size of the digest in bytes
+ *
+ **/
+size_t
+librdf_digest_get_digest_length(librdf_digest* digest)
+{
+  return digest->factory->digest_length;
+}
+
+
 /**
  * librdf_digest_to_string - Get a string representation of the digest object
  * @digest: the digest
@@ -334,7 +385,7 @@ librdf_digest_print(librdf_digest* digest, FILE* fh)
 
 
 /**
- * librdf_init_digest - Initialise the librdf_digest class
+ * librdf_init_digest - INTERNAL - Initialise the librdf_digest class
  * @world: redland world object
  **/
 void
@@ -361,7 +412,7 @@ librdf_init_digest(librdf_world *world)
 
 
 /**
- * librdf_finish_digest - Terminate the librdf_digest class
+ * librdf_finish_digest - INTERNAL - Terminate the librdf_digest class
  * @world: redland world object
  **/
 void
