@@ -2,9 +2,9 @@
  *
  * rdf_digest.h - RDF Digest Factory / Digest interfaces and definition
  *
- * $Id: rdf_digest.h,v 1.23 2004/11/01 20:43:33 cmdjb Exp $
+ * $Id: rdf_digest.h,v 1.27 2005/05/23 13:10:27 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -50,7 +50,7 @@ struct librdf_digest_factory_s
 
   /* functions (over context) */
   void (*init)( void *_context );
-  void (*update)( void *_context, unsigned char *buf, size_t nbytes );
+  void (*update)( void *_context, const unsigned char *buf, size_t nbytes );
   void (*final)( void *_context );
   unsigned char *(*get_digest)( void *_context );
 };
@@ -63,37 +63,10 @@ struct librdf_digest_s {
   librdf_digest_factory* factory;
 };
 
-#endif
-
-
-/* factory static methods */
-void librdf_digest_register_factory(librdf_world *world, const char *name, void (*factory) (librdf_digest_factory*));
-
-librdf_digest_factory* librdf_get_digest_factory(librdf_world *world, const char *name);
-
-
 /* module init */
 void librdf_init_digest(librdf_world *world);
 /* module finish */
 void librdf_finish_digest(librdf_world *world);
-                    
-/* constructor */
-librdf_digest* librdf_new_digest(librdf_world *world, char *name);
-librdf_digest* librdf_new_digest_from_factory(librdf_world *world, librdf_digest_factory *factory);
-
-/* destructor */
-void librdf_free_digest(librdf_digest *digest);
-
-
-/* methods */
-void librdf_digest_init(librdf_digest* digest);
-void librdf_digest_update(librdf_digest* digest, unsigned char *buf, size_t length);
-void librdf_digest_final(librdf_digest* digest);
-void* librdf_digest_get_digest(librdf_digest* digest);
-
-char* librdf_digest_to_string(librdf_digest* digest);
-void librdf_digest_print(librdf_digest* digest, FILE* fh);
-
 
 /* in librdf_digest_openssl.c */
 #ifdef HAVE_OPENSSL_DIGESTS
@@ -115,6 +88,34 @@ void librdf_digest_sha1_constructor(librdf_world *world);
 void librdf_digest_rmd160_constructor(librdf_world *world);
 #endif
 
+/* factory static methods */
+void librdf_digest_register_factory(librdf_world *world, const char *name, void (*factory) (librdf_digest_factory*));
+
+librdf_digest_factory* librdf_get_digest_factory(librdf_world *world, const char *name);
+
+/* constructor */
+librdf_digest* librdf_new_digest_from_factory(librdf_world *world, librdf_digest_factory *factory);
+
+#endif
+
+
+/* public constructor */
+REDLAND_API librdf_digest* librdf_new_digest(librdf_world *world, char *name);
+
+/* destructor */
+REDLAND_API void librdf_free_digest(librdf_digest *digest);
+
+
+/* methods */
+REDLAND_API void librdf_digest_init(librdf_digest* digest);
+REDLAND_API void librdf_digest_update(librdf_digest* digest, const unsigned char *buf, size_t length);
+REDLAND_API void librdf_digest_update_string(librdf_digest* digest, const unsigned char *string);
+REDLAND_API void librdf_digest_final(librdf_digest* digest);
+REDLAND_API void* librdf_digest_get_digest(librdf_digest* digest);
+REDLAND_API size_t librdf_digest_get_digest_length(librdf_digest* digest);
+
+REDLAND_API char* librdf_digest_to_string(librdf_digest* digest);
+REDLAND_API void librdf_digest_print(librdf_digest* digest, FILE* fh);
 
 #ifdef __cplusplus
 }
