@@ -1,10 +1,10 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdf_hash.c - RDF Hash Implementation
+ * rdf_hash.c - RDF Hash interface - set of (key: value) pairs with dups
  *
- * $Id: rdf_hash.c,v 1.79 2004/11/01 20:43:33 cmdjb Exp $
+ * $Id: rdf_hash.c,v 1.84 2005/05/23 14:17:42 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -78,7 +78,7 @@ static void librdf_hash_keys_iterator_finished(void* iterator);
 
 
 /**
- * librdf_init_hash - Initialise the librdf_hash module
+ * librdf_init_hash - INTERNAL - Initialise the librdf_hash module
  *
  * Initialises and registers all
  * compiled hash modules.  Must be called before using any of the hash
@@ -99,7 +99,7 @@ librdf_init_hash(librdf_world *world)
 
 
 /**
- * librdf_finish_hash - Terminate the librdf_hash module
+ * librdf_finish_hash - INTERNAL - Terminate the librdf_hash module
  * @world: redland world object
  **/
 void
@@ -295,7 +295,8 @@ librdf_get_hash_factory(librdf_world *world, const char *name)
  * Return value: a new &librdf_hash object or NULL on failure
  */
 librdf_hash*
-librdf_new_hash(librdf_world *world, char* name) {
+librdf_new_hash(librdf_world *world, const char* name)
+{
   librdf_hash_factory *factory;
 
   factory=librdf_get_hash_factory(world, name);
@@ -315,7 +316,8 @@ librdf_new_hash(librdf_world *world, char* name) {
  */
 librdf_hash*
 librdf_new_hash_from_factory (librdf_world *world,
-                              librdf_hash_factory* factory) {
+                              librdf_hash_factory* factory)
+{
   librdf_hash* h;
 
   h=(librdf_hash*)LIBRDF_CALLOC(librdf_hash, sizeof(librdf_hash), 1);
@@ -340,6 +342,62 @@ librdf_new_hash_from_factory (librdf_world *world,
 }
 
 
+/**
+ * librdf_new_hash_from_string -  Constructor - create a new librdf_hash object from a string
+ * @world: redland world object
+ * @name: hash name
+ * @string: hash encoded as a string
+ * 
+ * See &librdf_hash_from_string for the string format.
+ * 
+ * Return value: a new &librdf_hash object or NULL on failure
+ */
+librdf_hash*
+librdf_new_hash_from_string(librdf_world *world, const char *name,
+                            const char *string)
+{
+  librdf_hash* hash;
+
+  hash=librdf_new_hash(world, name);
+  if(!hash)
+    return NULL;
+  
+  if(librdf_hash_from_string(hash, string)) {
+    librdf_free_hash(hash);
+    return NULL;
+  }
+
+  return hash;
+}
+
+
+/**
+ * librdf_new_hash_from_array_of_strings -  Constructor - create a new librdf_hash object from an array of strings
+ * @world: redland world object
+ * @name: hash name
+ * @array: address of the start of the array of char* pointers
+ *
+ * Return value: a new &librdf_hash object or NULL on failure
+ */
+librdf_hash*
+librdf_new_hash_from_array_of_strings(librdf_world *world, const char *name,
+                                      const char **array)
+{
+  librdf_hash* hash;
+
+  hash=librdf_new_hash(world, name);
+  if(!hash)
+    return NULL;
+  
+  if(librdf_hash_from_array_of_strings(hash, array)) {
+    librdf_free_hash(hash);
+    return NULL;
+  }
+
+  return hash;
+}
+
+
 /**
  * librdf_new_hash_from_hash -  Copy Constructor - create a new librdf_hash object from an existing one
  * @old_hash: the hash to use to construct the hash
@@ -347,7 +405,8 @@ librdf_new_hash_from_factory (librdf_world *world,
  * Return value: a new &librdf_hash object or NULL on failure
  */
 librdf_hash*
-librdf_new_hash_from_hash(librdf_hash* old_hash) {
+librdf_new_hash_from_hash(librdf_hash* old_hash)
+{
   librdf_hash* hash;
   
   hash=(librdf_hash*)LIBRDF_CALLOC(librdf_hash, sizeof(librdf_hash), 1);
@@ -739,7 +798,7 @@ librdf_hash_get_all_iterator_finished(void* iterator)
  * Return value: the value or NULL on failure
  **/
 char*
-librdf_hash_get_del(librdf_hash* hash, char *key)
+librdf_hash_get_del(librdf_hash* hash, const char *key)
 {
   librdf_hash_datum hd_key;
   char *s;
@@ -748,7 +807,7 @@ librdf_hash_get_del(librdf_hash* hash, char *key)
   if(!s)
     return NULL;
 
-  hd_key.data=key;
+  hd_key.data=(char*)key;
   hd_key.size=strlen(key);
   
   librdf_hash_delete_all(hash, &hd_key);
@@ -1136,7 +1195,7 @@ typedef enum {
  * Return value: non 0 on failure
  **/
 int
-librdf_hash_from_string (librdf_hash* hash, const char *string) 
+librdf_hash_from_string(librdf_hash* hash, const char *string) 
 {
   const char * p;
   librdf_hash_datum hd_key, hd_value; /* on stack */
@@ -1322,13 +1381,13 @@ librdf_hash_from_string (librdf_hash* hash, const char *string)
  * Return value:  non 0 on failure
  **/
 int
-librdf_hash_from_array_of_strings (librdf_hash* hash, char **array) 
+librdf_hash_from_array_of_strings(librdf_hash* hash, const char **array)
 {
   librdf_hash_datum key, value; /* on stack */
   int i;
   
-  for(i=0; (key.data=array[i]); i+=2) {
-    value.data=array[i+1];
+  for(i=0; (key.data=(char*)array[i]); i+=2) {
+    value.data=(char*)array[i+1];
     if(!value.data) {
       librdf_log(hash->world, 
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_HASH, NULL,
@@ -1351,7 +1410,7 @@ librdf_hash_from_array_of_strings (librdf_hash* hash, char **array)
  * Return value: >0 (for true), 0 (for false) or <0 (for key not found or not known boolean value)
  **/
 int
-librdf_hash_get_as_boolean (librdf_hash* hash, char *key) 
+librdf_hash_get_as_boolean (librdf_hash* hash, const char *key) 
 {
   int bvalue= (-1);
   char *value;
@@ -1395,7 +1454,7 @@ librdf_hash_get_as_boolean (librdf_hash* hash, char *key)
  * Return value: >0 (for success), <0 (for key not found or not known boolean value)
  **/
 long
-librdf_hash_get_as_long (librdf_hash* hash, char *key) 
+librdf_hash_get_as_long (librdf_hash* hash, const char *key) 
 {
   int lvalue;
   char *value;
@@ -1428,7 +1487,8 @@ librdf_hash_get_as_long (librdf_hash* hash, char *key)
  * Return value: non 0 on failure
  **/
 int
-librdf_hash_put_strings(librdf_hash* hash, const char *key, const char *value) {
+librdf_hash_put_strings(librdf_hash* hash, const char *key, const char *value)
+{
   librdf_hash_datum key_hd; /* static */
   librdf_hash_datum value_hd;
 
@@ -1469,13 +1529,13 @@ main(int argc, char *argv[])
                             "colour", "yellow",
 			    NULL, NULL};
   char *test_duplicate_key="colour";
-  char *test_hash_array[]={"shape", "cube",
-			   "sides", "6", /* for testing get as long */
-                           "3d", "yes", /* testing bool */
-			   "colours", "red",
-			   "colours", "yellow",
-			   "creator", "rubik",
-			   NULL};
+  const char *test_hash_array[]={"shape", "cube",
+                                 "sides", "6", /* for testing get as long */
+                                 "3d", "yes", /* testing bool */
+                                 "colours", "red",
+                                 "colours", "yellow",
+                                 "creator", "rubik",
+                                 NULL};
   const char * const test_hash_string="field1='value1', field2='\\'value2', field3='\\\\', field4='\\\\\\'', field5 = 'a' ";
   char *test_hash_delete_key="size";
   int i,j;
