@@ -2,9 +2,9 @@
  *
  * rdf_hash.h - RDF Hash Factory and Hash interfaces and definitions
  *
- * $Id: rdf_hash.h,v 1.30 2004/11/01 20:43:33 cmdjb Exp $
+ * $Id: rdf_hash.h,v 1.34 2005/05/23 12:55:25 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -112,15 +112,6 @@ struct librdf_hash_factory_s {
 };
 typedef struct librdf_hash_factory_s librdf_hash_factory;
 
-#endif
-
-/* hash cursor_get method flags */
-#define LIBRDF_HASH_CURSOR_SET 0
-#define LIBRDF_HASH_CURSOR_NEXT_VALUE 1
-#define LIBRDF_HASH_CURSOR_FIRST 2
-#define LIBRDF_HASH_CURSOR_NEXT 3
-
-
 /* factory class methods */
 void librdf_hash_register_factory(librdf_world *world, const char *name, void (*factory) (librdf_hash_factory*));
 librdf_hash_factory* librdf_get_hash_factory(librdf_world *world, const char *name);
@@ -132,14 +123,16 @@ void librdf_init_hash(librdf_world *world);
 /* module terminate */
 void librdf_finish_hash(librdf_world *world);
 
+/* hash cursor_get method flags */
+#define LIBRDF_HASH_CURSOR_SET 0
+#define LIBRDF_HASH_CURSOR_NEXT_VALUE 1
+#define LIBRDF_HASH_CURSOR_FIRST 2
+#define LIBRDF_HASH_CURSOR_NEXT 3
+
+
 /* constructors */
-librdf_hash* librdf_new_hash(librdf_world *world, char *name);
+librdf_hash* librdf_new_hash(librdf_world *world, const char *name);
 librdf_hash* librdf_new_hash_from_factory(librdf_world *world, librdf_hash_factory* factory);
-librdf_hash* librdf_new_hash_from_hash (librdf_hash* old_hash);
-
-/* destructor */
-void librdf_free_hash(librdf_hash *hash);
-
 
 /* methods */
 
@@ -151,19 +144,14 @@ int librdf_hash_close(librdf_hash* hash);
 /* how many values */
 int librdf_hash_values_count(librdf_hash* hash);
 
-/* retrieve one value for a given hash key either as char or hash datum */
-char* librdf_hash_get(librdf_hash* hash, const char *key);
+/* retrieve one value for a given hash key as a hash datum */
 librdf_hash_datum* librdf_hash_get_one(librdf_hash* hash, librdf_hash_datum *key);
 
-/* retrieve one value for key and delete from hash all other values */
-char* librdf_hash_get_del(librdf_hash* hash, char *key);
-
 /* retrieve all values for a given hash key according to flags */
 librdf_iterator* librdf_hash_get_all(librdf_hash* hash, librdf_hash_datum *key, librdf_hash_datum *value);
 
 /* insert a key/value pair */
 int librdf_hash_put(librdf_hash* hash, librdf_hash_datum *key, librdf_hash_datum *value);
-int librdf_hash_put_strings(librdf_hash* hash, const char *key, const char *value);
 
   /* returns true if key exists in hash, without returning value */
 int librdf_hash_exists(librdf_hash* hash, librdf_hash_datum *key, librdf_hash_datum *value);
@@ -171,27 +159,17 @@ int librdf_hash_exists(librdf_hash* hash, librdf_hash_datum *key, librdf_hash_da
 int librdf_hash_delete(librdf_hash* hash, librdf_hash_datum *key, librdf_hash_datum *value);
 int librdf_hash_delete_all(librdf_hash* hash, librdf_hash_datum *key);
 librdf_iterator* librdf_hash_keys(librdf_hash* hash, librdf_hash_datum *key);
+
 /* flush any cached information to disk */
 int librdf_hash_sync(librdf_hash* hash);
 /* get the file descriptor for the hash, if it is file based (for locking) */
 int librdf_hash_get_fd(librdf_hash* hash);
 
-/* extra methods */
-void librdf_hash_print(librdf_hash* hash, FILE *fh);
-void librdf_hash_print_keys(librdf_hash* hash, FILE *fh);
-void librdf_hash_print_values(librdf_hash* hash, char *key, FILE *fh);
+/* init a hash from a string representation */
+int librdf_hash_from_string(librdf_hash* hash, const char *string);
 
-/* import a hash from a string representation */
-int librdf_hash_from_string (librdf_hash* hash, const char *string);
-
-/* import a hash from an array of strings */
-int librdf_hash_from_array_of_strings (librdf_hash* hash, char *array[]);
-
-/* lookup a hash key and decode value as a boolean */
-int librdf_hash_get_as_boolean (librdf_hash* hash, char *key);
-
-/* lookup a hash key and decode value as a long */
-long librdf_hash_get_as_long (librdf_hash* hash, char *key);
+/* init a hash from an array of strings */
+int librdf_hash_from_array_of_strings(librdf_hash* hash, const char *array[]);
 
 
 /* cursor methods from rdf_hash_cursor.c */
@@ -203,8 +181,40 @@ int librdf_hash_cursor_get_next_value(librdf_hash_cursor *cursor, librdf_hash_da
 int librdf_hash_cursor_get_first(librdf_hash_cursor *cursor, librdf_hash_datum *key, librdf_hash_datum *value);
 int librdf_hash_cursor_get_next(librdf_hash_cursor *cursor, librdf_hash_datum *key, librdf_hash_datum *value);
 
+#endif
 
 
+/* public constructors */
+REDLAND_API librdf_hash* librdf_new_hash_from_string(librdf_world *world, const char *name, const char *string);
+REDLAND_API librdf_hash* librdf_new_hash_from_array_of_strings(librdf_world *world, const char *name, const char **string);
+
+/* public copy constructor */
+REDLAND_API librdf_hash* librdf_new_hash_from_hash (librdf_hash* old_hash);
+
+/* public destructor */
+REDLAND_API void librdf_free_hash(librdf_hash *hash);
+
+/* public methods */
+
+/* retrieve one value for a given hash key */
+REDLAND_API char* librdf_hash_get(librdf_hash* hash, const char *key);
+
+/* lookup a hash key and decode value as a boolean */
+REDLAND_API int librdf_hash_get_as_boolean(librdf_hash* hash, const char *key);
+
+/* lookup a hash key and decode value as a long */
+REDLAND_API long librdf_hash_get_as_long(librdf_hash* hash, const char *key);
+
+/* retrieve one value for key and delete from hash all other values */
+REDLAND_API char* librdf_hash_get_del(librdf_hash* hash, const char *key);
+
+/* insert a key/value pair */
+REDLAND_API int librdf_hash_put_strings(librdf_hash* hash, const char *key, const char *value);
+
+REDLAND_API void librdf_hash_print(librdf_hash* hash, FILE *fh);
+REDLAND_API void librdf_hash_print_keys(librdf_hash* hash, FILE *fh);
+REDLAND_API void librdf_hash_print_values(librdf_hash* hash, char *key, FILE *fh);
+
 
 #ifdef __cplusplus
 }
