@@ -1,10 +1,10 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdf_init.c - Overall library initialisation / termination
+ * rdf_init.c - Redland library initialisation / termination
  *
- * $Id: rdf_init.c,v 1.62 2005/01/30 23:15:08 cmdjb Exp $
+ * $Id: rdf_init.c,v 1.65 2005/05/18 16:22:28 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -151,10 +151,8 @@ librdf_free_world(librdf_world *world)
 
 
 /**
- * librdf_world_init_mutex - Create the world mutex
+ * librdf_world_init_mutex - INTERNAL - Create the world mutex
  * @world: redland world object
- *
- * Internal
  */
 void
 librdf_world_init_mutex(librdf_world* world)
@@ -196,7 +194,6 @@ librdf_world_open(librdf_world *world)
 
   librdf_init_statement(world);
   librdf_init_model(world);
-  librdf_init_query(world);
   librdf_init_storage(world);
 
   /* NOTE: raptor is always initialised as a parser and may
@@ -206,6 +203,18 @@ librdf_world_open(librdf_world *world)
    */
   librdf_init_parser(world);
   librdf_init_serializer(world);
+
+  /* NOTE: Since initialising rasqal calls raptor to create URIs,
+   * rasqal must be initialised after raptor so that the raptor URI
+   * implementation is changed with raptor_uri_set_handler() in
+   * librdf_parser_raptor_constructor() before rasqal tries to create
+   * URIs.
+   *
+   * Otherwise URIs would be allocated with raptor as char* strings
+   * and then attempted to be freed later as librdf_uri*.  Which
+   * would be bad.
+   */
+  librdf_init_query(world);
 }
 
 
@@ -428,6 +437,75 @@ librdf_destroy_world(void)
   librdf_free_world(RDF_World);
 }
 
+
+#if defined (LIBRDF_DEBUG) && defined(LIBRDF_MEMORY_SIGN)
+void*
+librdf_sign_malloc(size_t size)
+{
+  int *p;
+  
+  size += sizeof(int);
+  
+  p=(int*)malloc(size);
+  *p++ = LIBRDF_SIGN_KEY;
+  return p;
+}
+
+void*
+librdf_sign_calloc(size_t nmemb, size_t size)
+{
+  int *p;
+  
+  /* turn into bytes */
+  size = nmemb*size + sizeof(int);
+  
+  p=(int*)calloc(1, size);
+  *p++ = LIBRDF_SIGN_KEY;
+  return p;
+}
+
+void*
+librdf_sign_realloc(void *ptr, size_t size)
+{
+  int *p;
+
+  if(!ptr)
+    return librdf_sign_malloc(size);
+  
+  p=(int*)ptr;
+  p--;
+
+  if(*p != LIBRDF_SIGN_KEY)
+    LIBRDF_FATAL1(NULL, LIBRDF_FROM_MEMORY, "memory signature wrong\n");
+
+  size += sizeof(int);
+  
+  p=(int*)realloc(p, size);
+  *p++= LIBRDF_SIGN_KEY;
+  return p;
+}
+
+void
+librdf_sign_free(void *ptr)
+{
+  int *p;
+
+  if(!ptr)
+    return;
+  
+  p=(int*)ptr;
+  p--;
+
+  if(*p != LIBRDF_SIGN_KEY)
+    LIBRDF_FATAL1(NULL, LIBRDF_FROM_MEMORY, "memory signature wrong");
+
+  free(p);
+}
+#endif
+
+
+
+
 #if defined (LIBRDF_DEBUG) && defined(HAVE_DMALLOC_H) && defined(LIBRDF_MEMORY_DEBUG_DMALLOC)
 
 #undef malloc
