@@ -2,7 +2,7 @@
  *
  * rdf_internal.h - Redland RDF Application Framework internal API (header never shipped)
  *
- * $Id: rdf_internal.h,v 1.14 2004/11/01 20:43:33 cmdjb Exp $
+ * $Id: rdf_internal.h,v 1.17 2005/05/20 15:04:54 cmdjb Exp $
  *
  * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -35,6 +35,11 @@
 #ifndef RDF_INTERNAL_H
 #define RDF_INTERNAL_H
 
+/* Can be over-ridden or undefined in a config.h file or -Ddefine */
+#ifndef REDLAND_INLINE
+#define REDLAND_INLINE inline
+#endif
+
 /* error handling */
 #ifdef LIBRDF_DEBUG
 /* DEBUGGING TURNED ON */
@@ -130,16 +135,28 @@ void librdf_system_free(void *ptr);
 #undef HAVE_STDLIB_H
 #endif
 
+#if defined(LIBRDF_MEMORY_SIGN)
+#define LIBRDF_SIGN_KEY 0x04Ed1A7D
+void* librdf_sign_malloc(size_t size);
+void* librdf_sign_calloc(size_t nmemb, size_t size);
+void* librdf_sign_realloc(void *ptr, size_t size);
+void librdf_sign_free(void *ptr);
+  
+#define LIBRDF_MALLOC(type, size)   librdf_sign_malloc(size)
+#define LIBRDF_CALLOC(type, nmemb, size) librdf_sign_calloc(nmemb, size)
+#define LIBRDF_REALLOC(type, ptr, size) librdf_sign_realloc(ptr, size)
+#define LIBRDF_FREE(type, ptr)   librdf_sign_free(ptr)
+
+#else
 #define LIBRDF_MALLOC(type, size) malloc(size)
 #define LIBRDF_CALLOC(type, size, count) calloc(size, count)
 #define LIBRDF_FREE(type, ptr)   free(ptr)
 
+#endif
 
 /* Fatal errors - always happen */
 #define LIBRDF_FATAL1(world, facility, message) librdf_fatal(world, facility, __FILE__, __LINE__ , __func__, message)
 #include <rdf_list.h>
-#include <rdf_hash.h>
-#include <rdf_digest.h>
 #include <rdf_files.h>
 #include <rdf_heuristics.h>
 
