@@ -1,8 +1,8 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdf_model.c - RDF Model implementation
+ * rdf_model.c - RDF Graph (Model) interface
  *
- * $Id: rdf_model.c,v 1.101 2005/01/30 23:15:12 cmdjb Exp $
+ * $Id: rdf_model.c,v 1.105 2005/05/20 14:25:13 cmdjb Exp $
  *
  * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -48,7 +48,7 @@ static void librdf_delete_model_factories(void);
 
 
 /**
- * librdf_init_model - Initialise librdf_model class
+ * librdf_init_model - INTERNAL - Initialise librdf_model class
  * @world: redland world object
  **/
 void
@@ -60,7 +60,7 @@ librdf_init_model(librdf_world *world)
 
 
 /**
- * librdf_finish_model - Terminate librdf_model class
+ * librdf_finish_model - INTERNAL - Terminate librdf_model class
  * @world: redland world object
  **/
 void
@@ -96,7 +96,7 @@ librdf_delete_model_factories(void)
 /*
  * librdf_model_supports_contexts - helper function to determine if this model supports contexts
  **/
-static inline int
+static REDLAND_INLINE int
 librdf_model_supports_contexts(librdf_model* model) {
   return model->supports_contexts;
 }
@@ -1315,7 +1315,7 @@ librdf_model_find_statements_in_context(librdf_model* model, librdf_statement* s
   stream=librdf_model_context_as_stream(model, context_node);
   if(!stream) {
     librdf_free_statement(statement);
-    return NULL;
+    return librdf_new_empty_stream(model->world);
   }
 
   librdf_stream_add_map(stream,
@@ -1624,10 +1624,10 @@ main(int argc, char *argv[])
   
   fprintf(stderr, "%s: Creating storage\n", program);
   if(1) {
-    /* test contexts in memory */
+    /* test in memory */
     storage=librdf_new_storage(world, NULL, NULL, "contexts='yes'");
   } else {
-    /* test contexts on disk */
+    /* test on disk */
     storage=librdf_new_storage(world, "hashes", "test", "hash-type='bdb',dir='.',write='yes',new='yes',contexts='yes'");
   }
   if(!storage) {
