@@ -1,10 +1,10 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdf_node.c - RDF Node (Resource / Literal) Implementation
+ * rdf_node.c - RDF Node (RDF URI, Literal, Blank Node) Interface
  *
- * $Id: rdf_node.c,v 1.103 2004/12/29 17:31:36 cmdjb Exp $
+ * $Id: rdf_node.c,v 1.106 2005/04/17 21:00:05 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -61,7 +61,7 @@ enum {
 /* class functions */
 
 /**
- * librdf_init_node - Initialise the node module.
+ * librdf_init_node - INTERNAL - Initialise the node module.
  * @world: redland world object
  * 
  **/
@@ -82,7 +82,7 @@ librdf_init_node(librdf_world* world)
 
 
 /**
- * librdf_finish_node - Terminate the librdf_node module
+ * librdf_finish_node - INTERNAL - Terminate the librdf_node module
  * @world: redland world object
  **/
 void
@@ -120,7 +120,7 @@ librdf_new_node(librdf_world *world)
 
     
 
-/*
+/**
  * librdf_new_node_from_uri_string_or_uri - INTERNAL - Constructor - create a new librdf_node object from a URI string or URI object
  * @world: redland world object
  * @uri_string: string representing a URI
@@ -666,7 +666,8 @@ librdf_free_node(librdf_node *node)
       if(node->value.blank.identifier != NULL)
         LIBRDF_FREE(cstring, node->value.blank.identifier);
       break;
-      
+
+    case LIBRDF_NODE_TYPE_UNKNOWN:
     default:
       break;
   }
@@ -720,7 +721,7 @@ librdf_node_get_type(librdf_node* node)
 #ifdef LIBRDF_DEBUG
 /* FIXME: For debugging purposes only */
 static const char* const librdf_node_type_names[] =
-{"Unknown", "Resource", "Literal", "<Reserved1>", "Blank"};
+{"Unknown", "Resource", "Literal", "<Unused1>", "Blank"};
 
 
 /*
@@ -1060,6 +1061,8 @@ librdf_node_to_counted_string(librdf_node* node, size_t* len_p)
       return NULL;
     sprintf((char*)s, "(%s)", node->value.blank.identifier);
     break;
+
+  case LIBRDF_NODE_TYPE_UNKNOWN:
   default:
       librdf_log(node->world,
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_NODE, NULL,
@@ -1131,6 +1134,9 @@ librdf_node_get_digest(librdf_node* node)
       librdf_digest_update(d, (unsigned char*)s, node->value.literal.string_len);
       librdf_digest_final(d);
       break;
+
+    case LIBRDF_NODE_TYPE_BLANK:
+    case LIBRDF_NODE_TYPE_UNKNOWN:
     default:
       librdf_log(world,
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_NODE, NULL,
@@ -1185,7 +1191,7 @@ librdf_node_encode(librdf_node* node, unsigned char *buffer, size_t length)
   unsigned char *string;
   size_t string_length;
   size_t language_length=0;
-  unsigned char *datatype_uri_string;
+  unsigned char *datatype_uri_string=NULL;
   size_t datatype_uri_length=0;
   
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(node, librdf_node, 0);
@@ -1289,6 +1295,7 @@ librdf_node_encode(librdf_node* node, unsigned char *buffer, size_t length)
       }
       break;
       
+    case LIBRDF_NODE_TYPE_UNKNOWN:
     default:
       librdf_log(node->world,
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_NODE, NULL,
