@@ -1,8 +1,8 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdf_query.c - RDF Query Adaptors Implementation
+ * rdf_query.c - RDF Query Language/Syntax and Execution Interface
  *
- * $Id: rdf_query.c,v 1.52 2005/01/30 23:15:21 cmdjb Exp $
+ * $Id: rdf_query.c,v 1.55 2005/05/20 14:25:25 cmdjb Exp $
  *
  * Copyright (C) 2002-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -53,7 +53,7 @@ static void librdf_delete_query_factories(librdf_world *world);
 
 
 /**
- * librdf_init_query - Initialise the librdf_query module
+ * librdf_init_query - INTERNAL - Initialise the librdf_query module
  * @world: redland world object
  * 
  * Initialises and registers all
@@ -70,7 +70,7 @@ librdf_init_query(librdf_world *world)
 
 
 /**
- * librdf_finish_query - Terminate the librdf_query module
+ * librdf_finish_query - INTERNAL - Terminate the librdf_query module
  * @world: redland world object
  **/
 void
@@ -434,6 +434,84 @@ librdf_query_execute(librdf_query* query, librdf_model* model)
   return results;
 }
 
+
+/**
+ * librdf_query_get_limit - Get the query-specified limit on results
+ * @query: &librdf_query query object
+ *
+ * This is the limit given in the query on the number of results allowed.
+ *
+ * Return value: integer >=0 if a limit is given, otherwise <0
+ **/
+int
+librdf_query_get_limit(librdf_query *query)
+{
+  if(query->factory->get_limit)
+    return query->factory->get_limit(query);
+  
+  return -1;
+}
+
+
+/**
+ * librdf_query_set_limit - Set the query-specified limit on results
+ * @query: &librdf_query query object
+ * @limit: the limit on results, >=0 to set a limit, <0 to have no limit
+ *
+ * This is the limit given in the query on the number of results allowed.
+ *
+ * Return value: non-0 on failure
+ **/
+int
+librdf_query_set_limit(librdf_query *query, int limit)
+{
+  if(query->factory->set_limit) {
+    query->factory->set_limit(query, limit);
+    return 0;
+  }
+
+  return -1;
+}
+
+
+/**
+ * librdf_query_get_offset - Get the query-specified offset on results
+ * @query: &librdf_query query object
+ *
+ * This is the offset given in the query on the number of results allowed.
+ *
+ * Return value: integer >=0 if a offset is given, otherwise <0
+ **/
+int
+librdf_query_get_offset(librdf_query *query)
+{
+  if(query->factory->get_offset)
+    return query->factory->get_offset(query);
+  
+  return -1;
+}
+
+
+/**
+ * librdf_query_set_offset - Set the query-specified offset on results
+ * @query: &librdf_query query object
+ * @offset: offset for results, >=0 to set an offset, <0 to have no offset
+ *
+ * This is the offset given in the query on the number of results allowed.
+ *
+ * Return value: non-0 on failure
+ **/
+int
+librdf_query_set_offset(librdf_query *query, int offset)
+{
+  if(query->factory->set_offset) {
+    query->factory->set_offset(query, offset);
+    return 0;
+  }
+
+  return -1;
+}
+
 #endif
 
 
@@ -477,7 +555,13 @@ main(int argc, char *argv[])
   librdf_world_open(world);
 
   /* create model and storage */
-  storage=librdf_new_storage(world, NULL, NULL, NULL);
+  if(1) {
+    /* test in memory */
+    storage=librdf_new_storage(world, NULL, NULL, NULL);
+  } else {
+    /* test on disk */
+    storage=librdf_new_storage(world, "hashes", "test", "hash-type='bdb',dir='.',write='yes',new='yes',contexts='yes'");
+  }
   if(!storage) {
     fprintf(stderr, "%s: Failed to create new storage\n", program);
     return(1);
