@@ -2,7 +2,7 @@
  *
  * rdf_query_rasqal.c - RDF Query with Rasqal
  *
- * $Id: rdf_query_rasqal.c,v 1.29 2005/02/01 00:16:25 cmdjb Exp $
+ * $Id: rdf_query_rasqal.c,v 1.34 2005/05/18 14:14:00 cmdjb Exp $
  *
  * Copyright (C) 2004-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -65,7 +65,7 @@ typedef struct
 
 
 /* prototypes for local functions */
-static rasqal_triples_match* rasqal_redland_new_triples_match(rasqal_triples_source *rts, void *user_data, rasqal_triple_meta *m, rasqal_triple *t);
+static int rasqal_redland_init_triples_match(rasqal_triples_match* rtm, rasqal_triples_source *rts, void *user_data, rasqal_triple_meta *m, rasqal_triple *t);
 static int rasqal_redland_triple_present(rasqal_triples_source *rts, void *user_data, rasqal_triple *t);
 static void rasqal_redland_free_triples_source(void *user_data);
 
@@ -198,12 +198,12 @@ redland_node_to_rasqal_literal(librdf_node *node) {
     size_t len;
 
     string=librdf_node_get_literal_value_as_counted_string(node, &len);
-    new_string=(unsigned char*)LIBRDF_MALLOC(cstring, len+1);
+    new_string=(unsigned char*)rasqal_alloc_memory(len+1);
     strcpy((char*)new_string, (const char*)string);
 
     string=(unsigned char*)librdf_node_get_literal_value_language(node);
     if(string) {
-      new_language=(char*)LIBRDF_MALLOC(cstring, strlen((const char*)string)+1);
+      new_language=(char*)rasqal_alloc_memory(strlen((const char*)string)+1);
       strcpy((char*)new_language, (const char*)string);
     }
     uri=librdf_node_get_literal_value_datatype_uri(node);
@@ -212,7 +212,7 @@ redland_node_to_rasqal_literal(librdf_node *node) {
     l=rasqal_new_string_literal((const unsigned char*)new_string, new_language, new_datatype, NULL);
   } else {
     unsigned char *blank=librdf_node_get_blank_identifier(node);
-    unsigned char *new_blank=(unsigned char*)LIBRDF_MALLOC(cstring, strlen((const char*)blank)+1);
+    unsigned char *new_blank=(unsigned char*)rasqal_alloc_memory(strlen((const char*)blank)+1);
     strcpy((char*)new_blank, (const char*)blank);
     l=rasqal_new_simple_literal(RASQAL_LITERAL_BLANK, (const unsigned char*)new_blank);
   }
@@ -239,7 +239,7 @@ rasqal_redland_new_triples_source(rasqal_query* rdf_query,
   raptor_sequence *seq;
   librdf_query_rasqal_context *context;
 
-  seq=rasqal_query_get_source_sequence(rdf_query);
+  seq=rasqal_query_get_data_graph_sequence(rdf_query);
   
   /* FIXME: queries with triple sources are actively discarded */
   if(seq && raptor_sequence_size(seq))
@@ -251,7 +251,7 @@ rasqal_redland_new_triples_source(rasqal_query* rdf_query,
   context=(librdf_query_rasqal_context*)rtsc->query->context;
   rtsc->model=context->model;
 
-  rts->new_triples_match=rasqal_redland_new_triples_match;
+  rts->init_triples_match=rasqal_redland_init_triples_match;
   rts->triple_present=rasqal_redland_triple_present;
   rts->free_triples_source=rasqal_redland_free_triples_source;
 
@@ -429,15 +429,14 @@ rasqal_redland_finish_triples_match(struct rasqal_triples_match_s* rtm,
 }
 
 
-static rasqal_triples_match*
-rasqal_redland_new_triples_match(rasqal_triples_source *rts, void *user_data,
-                                 rasqal_triple_meta *m, rasqal_triple *t) {
+static int
+rasqal_redland_init_triples_match(rasqal_triples_match* rtm,
+                                  rasqal_triples_source *rts, void *user_data,
+                                  rasqal_triple_meta *m, rasqal_triple *t) {
   rasqal_redland_triples_source_user_data* rtsc=(rasqal_redland_triples_source_user_data*)user_data;
-  rasqal_triples_match *rtm;
   rasqal_redland_triples_match_context* rtmc;
   rasqal_variable* var;
 
-  rtm=(rasqal_triples_match *)LIBRDF_CALLOC(rasqal_triples_match, sizeof(rasqal_triples_match), 1);
   rtm->bind_match=rasqal_redland_bind_match;
   rtm->next_match=rasqal_redland_next_match;
   rtm->is_end=rasqal_redland_is_end;
@@ -503,7 +502,7 @@ rasqal_redland_new_triples_match(rasqal_triples_source *rts, void *user_data,
                                                    rtmc->nodes[1], 
                                                    rtmc->nodes[2]);
   if(!rtmc->qstatement)
-    return NULL;
+    return 1;
 
 #ifdef RASQAL_DEBUG
   LIBRDF_DEBUG1("query statement: ");
@@ -513,9 +512,9 @@ rasqal_redland_new_triples_match(rasqal_triples_source *rts, void *user_data,
   
   rtmc->stream=librdf_model_find_statements(rtsc->model, rtmc->qstatement);
 
-  LIBRDF_DEBUG1("rasqal_new_triples_match done\n");
+  LIBRDF_DEBUG1("rasqal_init_triples_match done\n");
 
-  return rtm;
+  return 0;
 }
 
 
@@ -532,6 +531,9 @@ librdf_query_rasqal_execute(librdf_query* query, librdf_model* model)
                           (raptor_uri*)context->uri))
     return NULL;
 
+  if(context->results)
+    rasqal_free_query_results(context->results);
+  
   context->results=rasqal_query_execute(context->rq);
   
   results=(librdf_query_results*)LIBRDF_MALLOC(librdf_query_results, sizeof(librdf_query_results));
@@ -541,6 +543,40 @@ librdf_query_rasqal_execute(librdf_query* query, librdf_model* model)
 }
 
 
+static int
+librdf_query_rasqal_get_limit(librdf_query* query)
+{
+  librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+  return rasqal_query_get_limit(context->rq);
+}
+
+
+static int
+librdf_query_rasqal_set_limit(librdf_query* query, int limit)
+{
+  librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+  rasqal_query_set_limit(context->rq, limit);
+  return 0;
+}
+
+
+static int
+librdf_query_rasqal_get_offset(librdf_query* query)
+{
+  librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+  return rasqal_query_get_offset(context->rq);
+}
+
+
+static int
+librdf_query_rasqal_set_offset(librdf_query* query, int offset)
+{
+  librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+  rasqal_query_set_offset(context->rq, offset);
+  return 0;
+}
+
+
 static int
 librdf_query_rasqal_results_get_count(librdf_query_results *query_results)
 {
@@ -670,6 +706,7 @@ librdf_query_rasqal_free_results(librdf_query_results* query_results)
     return;
   
   rasqal_free_query_results(context->results);
+  context->results=NULL;
 }
 
 
@@ -993,6 +1030,11 @@ librdf_query_rasqal_register_factory(librdf_query_factory *factory)
   factory->init               = librdf_query_rasqal_init;
   factory->terminate          = librdf_query_rasqal_terminate;
   factory->execute            = librdf_query_rasqal_execute;
+  factory->get_limit          = librdf_query_rasqal_get_limit;
+  factory->set_limit          = librdf_query_rasqal_set_limit;
+  factory->get_offset         = librdf_query_rasqal_get_offset;
+  factory->set_offset         = librdf_query_rasqal_set_offset;
+
   factory->results_get_count           = librdf_query_rasqal_results_get_count;
   factory->results_next                = librdf_query_rasqal_results_next;
   factory->results_finished            = librdf_query_rasqal_results_finished;
@@ -1009,7 +1051,6 @@ librdf_query_rasqal_register_factory(librdf_query_factory *factory)
   factory->results_is_graph                   = librdf_query_rasqal_results_is_graph;
   factory->results_get_boolean                = librdf_query_rasqal_results_get_boolean;
   factory->results_as_stream                  = librdf_query_rasqal_results_as_stream;
-
 }
 
 
