@@ -1,8 +1,8 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdf_serializer.c - RDF Serializer Factory implementation
+ * rdf_serializer.c - RDF Serializer (RDF triples to syntax) interface
  *
- * $Id: rdf_serializer.c,v 1.42 2005/01/30 23:15:25 cmdjb Exp $
+ * $Id: rdf_serializer.c,v 1.44 2005/04/17 20:07:41 cmdjb Exp $
  *
  * Copyright (C) 2002-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -430,7 +430,7 @@ librdf_serializer_serialize_model_to_string(librdf_serializer* serializer,
 
 
 /**
- * librdf_init_serializer - Initialise the librdf_serializer class
+ * librdf_init_serializer - INTERNAL - Initialise the librdf_serializer class
  * @world: redland world object
  **/
 void
@@ -441,7 +441,7 @@ librdf_init_serializer(librdf_world *world)
 
 
 /**
- * librdf_finish_serializer - Terminate the librdf_serializer class
+ * librdf_finish_serializer - INTERNAL - Terminate the librdf_serializer class
  * @world: redland world object
  **/
 void
@@ -580,6 +580,11 @@ log_handler(void *user_data, librdf_log_message *message)
     case LIBRDF_LOG_WARN:
       ld->warnings++;
       break;
+
+    case LIBRDF_LOG_NONE:
+    case LIBRDF_LOG_DEBUG:
+    case LIBRDF_LOG_INFO:
+    case LIBRDF_LOG_FATAL:
     default:
       break;
   }
