@@ -1,10 +1,10 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdf_statement.c - RDF Statement implementation
+ * rdf_statement.c - RDF Triple (Statement) interface
  *
- * $Id: rdf_statement.c,v 1.73 2004/11/01 20:43:34 cmdjb Exp $
+ * $Id: rdf_statement.c,v 1.75 2005/04/17 21:00:09 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -48,7 +48,7 @@
 /* class methods */
 
 /**
- * librdf_init_statement - Initialise the librdf_statement module
+ * librdf_init_statement - INTERNAL - Initialise the librdf_statement module
  * @world: redland world object
  **/
 void
@@ -58,7 +58,7 @@ librdf_init_statement(librdf_world *world)
 
 
 /**
- * librdf_finish_statement - Terminate the librdf_statement module
+ * librdf_finish_statement - INTERNAL - Terminate the librdf_statement module
  * @world: redland world object
  **/
 void
@@ -236,7 +236,7 @@ librdf_free_statement(librdf_statement* statement)
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN(statement, librdf_statement);
 
 #ifdef WITH_THREADS
-  world = node->world;
+  world = statement->world;
   pthread_mutex_lock(world->statements_mutex);
 #endif
 
