@@ -1,10 +1,10 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdf_storage.c - RDF Storage Implementation
+ * rdf_storage.c - RDF Storage (Triple store) interface
  *
- * $Id: rdf_storage.c,v 1.81 2004/11/22 23:59:27 cmdjb Exp $
+ * $Id: rdf_storage.c,v 1.86 2005/05/18 16:05:31 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -65,7 +65,7 @@ static librdf_iterator* librdf_storage_node_stream_to_node_create(librdf_storage
 
 
 /**
- * librdf_init_storage - Initialise the librdf_storage module
+ * librdf_init_storage - INTERNAL - Initialise the librdf_storage module
  * @world: redland world object
  * 
  * Initialises and registers all
@@ -95,7 +95,7 @@ librdf_init_storage(librdf_world *world)
 
 
 /**
- * librdf_finish_storage - Terminate the librdf_storage module
+ * librdf_finish_storage - INTERNAL - Terminate the librdf_storage module
  * @world: redland world object
  **/
 void
@@ -775,6 +775,7 @@ librdf_storage_stream_to_node_iterator_get_method(void* iterator, int flags)
           node=librdf_statement_get_object(statement);
           break;
 
+        case LIBRDF_STATEMENT_ALL:
         default: /* error */
           librdf_log(statement->world,
                      0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
@@ -826,9 +827,11 @@ librdf_storage_stream_to_node_iterator_finished(void* iterator)
 /*
  * librdf_storage_node_stream_to_node_create - Create a stream for get sources, targets or arcs methods using find_statements method
  * @storage: the storage object to use
- * @node1: the first node to encode in the key
+ * @node1: the first node to encode in the key (or NULL if not needed)
  * @node2: the second node to encode in the key (or NULL if not needed)
  * @want: the field required from the statement
+ *
+ * node1 and node2 cannot both be NULL
  * 
  * Return value: a new &librdf_iterator or NULL on failure
  **/
@@ -844,7 +847,7 @@ librdf_storage_node_stream_to_node_create(librdf_storage* storage,
   librdf_iterator *iterator;
   
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, NULL);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(node1, librdf_node, NULL);
+  LIBRDF_ASSERT_RETURN(node1 == NULL && node2 == NULL, "both node objects are NULL", NULL);
 
   partial_statement=librdf_new_statement(storage->world);
   if(!partial_statement)
@@ -869,6 +872,8 @@ librdf_storage_node_stream_to_node_create(librdf_storage* storage,
       librdf_statement_set_subject(partial_statement, node1);
       librdf_statement_set_predicate(partial_statement, node2);
       break;
+
+    case LIBRDF_STATEMENT_ALL:
     default:
       librdf_free_statement(partial_statement);
       librdf_log(storage->world,
@@ -880,7 +885,7 @@ librdf_storage_node_stream_to_node_create(librdf_storage* storage,
   stream=storage->factory->find_statements(storage, partial_statement);
   if(!stream) {
     librdf_storage_stream_to_node_iterator_finished(context);
-    return NULL;
+    return librdf_new_empty_iterator(storage->world);
   }
   
   /* initialise context */
