@@ -2,7 +2,7 @@
  *
  * rdf_storage_hashes.c - RDF Storage as Hashes Implementation
  *
- * $Id: rdf_storage_hashes.c,v 1.76 2004/11/10 10:21:31 cmdjb Exp $
+ * $Id: rdf_storage_hashes.c,v 1.79 2005/05/18 16:05:32 cmdjb Exp $
  *
  * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -827,7 +827,7 @@ librdf_storage_hashes_serialise_common(librdf_storage* storage, int hash_index,
   }
   if(!scontext->iterator) {
     librdf_storage_hashes_serialise_finished((void*)scontext);
-    return NULL;
+    return librdf_new_empty_stream(storage->world);
   }
 
   scontext->storage=storage;
@@ -1319,7 +1319,7 @@ librdf_storage_hashes_node_iterator_create(librdf_storage* storage,
   if(!icontext->iterator) {
     LIBRDF_FREE(data, key_buffer);
     librdf_storage_hashes_node_iterator_finished(icontext);
-    return NULL;
+    return librdf_new_empty_iterator(storage->world);
   }
 
   LIBRDF_FREE(data, key_buffer);
@@ -1521,10 +1521,8 @@ librdf_storage_hashes_context_serialise(librdf_storage* storage,
 
   scontext->iterator=librdf_hash_get_all(context->hashes[context->contexts_index], 
                                          scontext->key, scontext->value);
-  if(!scontext->iterator) {
-    librdf_storage_hashes_context_serialise_finished(scontext);
-    return NULL;
-  }
+  if(!scontext->iterator)
+    return librdf_new_empty_stream(storage->world);
 
 
   scontext->storage=storage;
