@@ -2,7 +2,7 @@
  *
  * rdf_storage_list.c - RDF Storage in memory as a list implementation
  *
- * $Id: rdf_storage_list.c,v 1.49 2004/11/01 20:43:35 cmdjb Exp $
+ * $Id: rdf_storage_list.c,v 1.53 2005/05/18 16:05:34 cmdjb Exp $
  *
  * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -147,6 +147,9 @@ librdf_storage_list_node_equals(librdf_storage_list_node *first,
   if(!first->context && !second->context)
     return 1;
   
+  if(!first->context || !second->context)
+    return 0;
+  
   if(!librdf_node_equals(first->context, second->context))
     return 0;
 
@@ -345,10 +348,8 @@ librdf_storage_list_serialise(librdf_storage* storage)
 
   scontext->index_contexts=context->index_contexts;
   scontext->iterator=librdf_list_get_iterator(context->list);
-  if(!scontext->iterator) {
-    librdf_storage_list_serialise_finished((void*)scontext);
-    return NULL;
-  }
+  if(!scontext->iterator)
+    return librdf_new_empty_stream(storage->world);
     
   
   scontext->storage=storage;
@@ -646,10 +647,8 @@ librdf_storage_list_context_serialise(librdf_storage* storage,
 
   scontext->iterator=librdf_hash_get_all(context->contexts, 
                                          scontext->key, scontext->value);
-  if(!scontext->iterator) {
-    librdf_storage_list_context_serialise_finished(scontext);
-    return NULL;
-  }
+  if(!scontext->iterator)
+    return librdf_new_empty_stream(storage->world);
 
 
   scontext->storage=storage;
@@ -873,7 +872,7 @@ librdf_storage_list_get_contexts(librdf_storage* storage)
   icontext->iterator=librdf_hash_keys(context->contexts, icontext->key);
   if(!icontext->iterator) {
     librdf_storage_list_get_contexts_finished(icontext);
-    return NULL;
+    return librdf_new_empty_iterator(storage->world);
   }
 
 
@@ -883,11 +882,9 @@ librdf_storage_list_get_contexts(librdf_storage* storage)
                                &librdf_storage_list_get_contexts_next_method,
                                &librdf_storage_list_get_contexts_get_method,
                                &librdf_storage_list_get_contexts_finished);
-  if(!iterator) {
+  if(!iterator)
     librdf_storage_list_get_contexts_finished((void*)icontext);
-    return NULL;
-  }
-  
+
   return iterator;  
 }
 
