@@ -2,7 +2,7 @@
  *
  * rdf_storage_sqlite.c - RDF Storage using SQLite implementation
  *
- * $Id: rdf_storage_sqlite.c,v 1.35 2005/01/30 23:15:30 cmdjb Exp $
+ * $Id: rdf_storage_sqlite.c,v 1.40 2005/05/20 14:07:54 cmdjb Exp $
  *
  * Copyright (C) 2004-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -593,6 +593,7 @@ librdf_storage_sqlite_node_helper(librdf_storage* storage,
       node_type=TRIPLE_BLANK;
       break;
 
+    case LIBRDF_NODE_TYPE_UNKNOWN:
     default:
       librdf_log(node->world,
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
@@ -1061,17 +1062,18 @@ librdf_storage_sqlite_serialise(librdf_storage* storage)
                         &scontext->vm,
                         &errmsg);
 #endif
-  raptor_free_stringbuffer(sb);
 
   if(status != SQLITE_OK) {
     librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                "SQLite database %s SQL compile failed - %s (%d)", 
                context->name, errmsg, status);
 
+    raptor_free_stringbuffer(sb);
     librdf_storage_sqlite_serialise_finished((void*)scontext);
     return NULL;
   }
-    
+  raptor_free_stringbuffer(sb);
+  
   scontext->storage=storage;
   librdf_storage_add_reference(scontext->storage);
 
@@ -1254,20 +1256,23 @@ static int
 librdf_storage_sqlite_serialise_end_of_stream(void* context)
 {
   librdf_storage_sqlite_serialise_stream_context* scontext=(librdf_storage_sqlite_serialise_stream_context*)context;
-  int result;
   
   if(scontext->finished)
     return 1;
   
-  result=librdf_storage_sqlite_get_next_common(scontext->sqlite_context,
-                                               scontext->vm,
-                                               &scontext->statement,
-                                               &scontext->context);
-  if(result) {
-    /* error or finished */
-    if(result < 0)
-      scontext->vm=NULL;
-    scontext->finished=1;
+  if(scontext->statement == NULL) {
+    int result;
+
+    result=librdf_storage_sqlite_get_next_common(scontext->sqlite_context,
+                                                 scontext->vm,
+                                                 &scontext->statement,
+                                                 &scontext->context);
+    if(result) {
+      /* error or finished */
+      if(result < 0)
+        scontext->vm=NULL;
+      scontext->finished=1;
+    }
   }
 
   return scontext->finished;
@@ -1280,6 +1285,9 @@ librdf_storage_sqlite_serialise_next_statement(void* context)
   librdf_storage_sqlite_serialise_stream_context* scontext=(librdf_storage_sqlite_serialise_stream_context*)context;
   int result;
   
+  if(scontext->finished)
+    return 1;
+  
   result=librdf_storage_sqlite_get_next_common(scontext->sqlite_context,
                                                scontext->vm,
                                                &scontext->statement,
@@ -1428,10 +1436,10 @@ librdf_storage_sqlite_find_statements(librdf_storage* storage,
       raptor_stringbuffer_append_counted_string(sb, 
                                                 (unsigned char*)" WHERE ", 7, 1);
       need_where=0;
-    }
-    if(need_and)
+      need_and=1;
+    } else if(need_and)
       raptor_stringbuffer_append_counted_string(sb, 
-                                                (unsigned char*)"AND ", 5, 1);
+                                                (unsigned char*)" AND ", 5, 1);
     raptor_stringbuffer_append_counted_string(sb, 
                                               (unsigned char*)"T.", 2, 1);
     raptor_stringbuffer_append_string(sb, fields[i], 1);
@@ -1440,7 +1448,6 @@ librdf_storage_sqlite_find_statements(librdf_storage* storage,
     raptor_stringbuffer_append_decimal(sb, node_ids[i]);
     raptor_stringbuffer_append_counted_string(sb, 
                                               (unsigned char*)"\n", 1, 1);
-    need_and=1;
   }
   raptor_stringbuffer_append_counted_string(sb, 
                                             (unsigned char*)";", 1, 1);
@@ -1465,16 +1472,17 @@ librdf_storage_sqlite_find_statements(librdf_storage* storage,
                         &scontext->vm,
                         &errmsg);
 #endif
-  raptor_free_stringbuffer(sb);
 
   if(status != SQLITE_OK) {
     librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                "SQLite database %s SQL compile '%s' failed - %s (%d)", 
                context->name, request, errmsg, status);
 
+    raptor_free_stringbuffer(sb);
     librdf_storage_sqlite_serialise_finished((void*)scontext);
     return NULL;
   }
+  raptor_free_stringbuffer(sb);
   
   scontext->storage=storage;
   librdf_storage_add_reference(scontext->storage);
@@ -1498,21 +1506,24 @@ static int
 librdf_storage_sqlite_find_statements_end_of_stream(void* context)
 {
   librdf_storage_sqlite_find_statements_stream_context* scontext=(librdf_storage_sqlite_find_statements_stream_context*)context;
-  int result;
   
   if(scontext->finished)
     return 1;
   
-  result=librdf_storage_sqlite_get_next_common(scontext->sqlite_context,
-                                               scontext->vm,
-                                               &scontext->statement,
-                                               &scontext->context);
-  if(result) {
-    /* error or finished */
-    if(result<0)
-      scontext->vm=NULL;
-    scontext->finished=1;
+  if(scontext->statement == NULL) {
+    int result;
+    result=librdf_storage_sqlite_get_next_common(scontext->sqlite_context,
+                                                 scontext->vm,
+                                                 &scontext->statement,
+                                                 &scontext->context);
+    if(result) {
+      /* error or finished */
+      if(result<0)
+        scontext->vm=NULL;
+      scontext->finished=1;
+    }
   }
+  
 
   return scontext->finished;
 }
@@ -1524,6 +1535,9 @@ librdf_storage_sqlite_find_statements_next_statement(void* context)
   librdf_storage_sqlite_find_statements_stream_context* scontext=(librdf_storage_sqlite_find_statements_stream_context*)context;
   int result;
   
+  if(scontext->finished)
+    return 1;
+  
   result=librdf_storage_sqlite_get_next_common(scontext->sqlite_context,
                                                scontext->vm,
                                                &scontext->statement,
@@ -1800,16 +1814,17 @@ librdf_storage_sqlite_context_serialise(librdf_storage* storage,
                         &scontext->vm,
                         &errmsg);
 #endif
-  raptor_free_stringbuffer(sb);
 
   if(status != SQLITE_OK) {
     librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                "SQLite database %s SQL compile failed - %s (%d)", 
                context->name, errmsg, status);
 
+    raptor_free_stringbuffer(sb);
     librdf_storage_sqlite_serialise_finished((void*)scontext);
     return NULL;
   }
+  raptor_free_stringbuffer(sb);
 
   scontext->storage=storage;
   librdf_storage_add_reference(scontext->storage);
@@ -1833,21 +1848,25 @@ static int
 librdf_storage_sqlite_context_serialise_end_of_stream(void* context)
 {
   librdf_storage_sqlite_context_serialise_stream_context* scontext=(librdf_storage_sqlite_context_serialise_stream_context*)context;
-  int result;
   
   if(scontext->finished)
     return 1;
   
-  result=librdf_storage_sqlite_get_next_common(scontext->sqlite_context,
-                                               scontext->vm,
-                                               &scontext->statement,
-                                               &scontext->context);
-  if(result) {
-    /* error or finished */
-    if(result < 0)
-      scontext->vm=NULL;
-    scontext->finished=1;
+  if(scontext->statement == NULL) {
+    int result;
+
+    result=librdf_storage_sqlite_get_next_common(scontext->sqlite_context,
+                                                 scontext->vm,
+                                                 &scontext->statement,
+                                                 &scontext->context);
+    if(result) {
+      /* error or finished */
+      if(result < 0)
+        scontext->vm=NULL;
+      scontext->finished=1;
+    }
   }
+  
 
   return scontext->finished;
 }
@@ -1859,6 +1878,9 @@ librdf_storage_sqlite_context_serialise_next_statement(void* context)
   librdf_storage_sqlite_context_serialise_stream_context* scontext=(librdf_storage_sqlite_context_serialise_stream_context*)context;
   int result;
   
+  if(scontext->finished)
+    return 1;
+  
   result=librdf_storage_sqlite_get_next_common(scontext->sqlite_context,
                                                scontext->vm,
                                                &scontext->statement,
@@ -2044,20 +2066,24 @@ static int
 librdf_storage_sqlite_get_contexts_is_end(void* iterator)
 {
   librdf_storage_sqlite_get_contexts_iterator_context* icontext=(librdf_storage_sqlite_get_contexts_iterator_context*)iterator;
-  int result;
   
   if(icontext->finished)
     return 1;
-  
-  result=librdf_storage_sqlite_get_next_context_common(icontext->sqlite_context,
-                                                       icontext->vm,
-                                                       &icontext->current);
-  if(result) {
-    /* error or finished */
-    if(result < 0)
-      icontext->vm=NULL;
-    icontext->finished=1;
+
+  if(!icontext->current) {
+    int result;
+    
+    result=librdf_storage_sqlite_get_next_context_common(icontext->sqlite_context,
+                                                         icontext->vm,
+                                                         &icontext->current);
+    if(result) {
+      /* error or finished */
+      if(result < 0)
+        icontext->vm=NULL;
+      icontext->finished=1;
+    }
   }
+  
 
   return icontext->finished;
 }
@@ -2069,6 +2095,9 @@ librdf_storage_sqlite_get_contexts_next_method(void* iterator)
   librdf_storage_sqlite_get_contexts_iterator_context* icontext=(librdf_storage_sqlite_get_contexts_iterator_context*)iterator;
   int result;
   
+  if(icontext->finished)
+    return 1;
+
   result=librdf_storage_sqlite_get_next_context_common(icontext->sqlite_context,
                                                        icontext->vm,
                                                        &icontext->current);
@@ -2201,16 +2230,17 @@ librdf_storage_sqlite_get_contexts(librdf_storage* storage)
                         &icontext->vm,
                         &errmsg);
 #endif
-  raptor_free_stringbuffer(sb);
 
   if(status != SQLITE_OK) {
     librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                "SQLite database %s SQL compile failed - %s (%d)", 
                context->name, errmsg, status);
 
+    raptor_free_stringbuffer(sb);
     librdf_storage_sqlite_get_contexts_finished((void*)icontext);
     return NULL;
   }
+  raptor_free_stringbuffer(sb);
     
   icontext->storage=storage;
   librdf_storage_add_reference(icontext->storage);
@@ -2221,10 +2251,8 @@ librdf_storage_sqlite_get_contexts(librdf_storage* storage)
                                &librdf_storage_sqlite_get_contexts_next_method,
                                &librdf_storage_sqlite_get_contexts_get_method,
                                &librdf_storage_sqlite_get_contexts_finished);
-  if(!iterator) {
+  if(!iterator)
     librdf_storage_sqlite_get_contexts_finished((void*)icontext);
-    return NULL;
-  }
   
   return iterator;  
 }
