@@ -1,10 +1,10 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdf_uri.c - RDF URI Implementation
+ * rdf_uri.c - RDF URI interface
  *
- * $Id: rdf_uri.c,v 1.68 2004/12/08 15:58:00 cmdjb Exp $
+ * $Id: rdf_uri.c,v 1.70 2005/05/23 13:19:20 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -53,7 +53,7 @@
 
 
 /**
- * librdf_init_uri - Initialise the librdf_uri class
+ * librdf_init_uri - INTERNAL - Initialise the librdf_uri class
  * @world: redland world object
  *
  **/
@@ -77,7 +77,7 @@ librdf_init_uri(librdf_world *world)
 
 
 /**
- * librdf_finish_uri - Terminate the librdf_uri class
+ * librdf_finish_uri - INTERNAL - Terminate the librdf_uri class
  * @world: redland world object
  **/
 void
@@ -473,7 +473,7 @@ librdf_uri_as_counted_string(librdf_uri *uri, size_t* len_p)
  * Return value: new &librdf_digest object or NULL on failure.
  **/
 librdf_digest*
-librdf_uri_get_digest (librdf_uri* uri) 
+librdf_uri_get_digest(librdf_uri* uri) 
 {
   librdf_world *world=uri->world;
   librdf_digest* d;
@@ -484,7 +484,6 @@ librdf_uri_get_digest (librdf_uri* uri)
   if(!d)
     return NULL;
   
-  librdf_digest_init(d);
   librdf_digest_update(d, (unsigned char*)uri->string, uri->string_length);
   librdf_digest_final(d);
   
