@@ -2,9 +2,9 @@
  *
  * raptor_identifier.c - Raptor identifier classes
  *
- * $Id: raptor_identifier.c,v 1.12 2004/09/20 11:54:19 cmdjb Exp $
+ * $Id: raptor_identifier.c,v 1.15 2005/05/17 22:46:08 cmdjb Exp $
  *
- * Copyright (C) 2003-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2003-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -179,7 +179,7 @@ raptor_free_identifier(raptor_identifier *identifier)
   }
 
   if(identifier->literal) {
-    RAPTOR_FREE(cstring, identifier->literal);
+    RAPTOR_FREE(cstring, (void*)identifier->literal);
     identifier->literal=NULL;
   }
 
@@ -189,7 +189,7 @@ raptor_free_identifier(raptor_identifier *identifier)
   }
 
   if(identifier->literal_language) {
-    RAPTOR_FREE(cstring, identifier->literal_language);
+    RAPTOR_FREE(cstring, (void*)identifier->literal_language);
     identifier->literal_language=NULL;
   }
 
@@ -216,7 +216,7 @@ raptor_identifier_print(FILE *stream, raptor_identifier *identifier)
       fprintf(stream, "@%s", identifier->literal_language);
     fputc('<', stream);
     if(identifier->type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL)
-      fputs(raptor_xml_literal_datatype_uri_string, stream);
+      fputs((const char*)raptor_xml_literal_datatype_uri_string, stream);
     else if(identifier->literal_datatype)
       fputs((const char*)raptor_uri_as_string(identifier->literal_datatype), stream);
     fputc('>', stream);
