@@ -2,7 +2,7 @@
  *
  * raptor_iostream.c - Raptor I/O-stream class for abstracting I/O
  *
- * $Id: raptor_iostream.c,v 1.18 2004/12/17 22:11:00 cmdjb Exp $
+ * $Id: raptor_iostream.c,v 1.20 2005/04/16 17:17:21 cmdjb Exp $
  *
  * Copyright (C) 2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -255,12 +255,13 @@ raptor_string_iostream_finish(void *context)
   if(con->length_p)
     *con->length_p=len;
   
-  if(len) {
-    str=(void*)con->malloc_handler(len+1);
-    if(str) {
+  str=(void*)con->malloc_handler(len+1);
+  if(str) {
+    if(len)
       raptor_stringbuffer_copy_to_string(con->sb, (unsigned char*)str, len+1);
-      *con->string_p=str;
-    }
+    else
+      *(char*)str='\0';
+    *con->string_p=str;
   }
 
   if(!str && con->length_p)
