@@ -2,7 +2,7 @@
  *
  * raptor_xml.c - Raptor XML routines
  *
- * $Id: raptor_xml.c,v 1.19 2004/11/01 17:53:27 cmdjb Exp $
+ * $Id: raptor_xml.c,v 1.23 2005/02/10 22:13:05 cmdjb Exp $
  *
  * Copyright (C) 2003-2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -68,38 +68,14 @@
 int
 raptor_valid_xml_ID(raptor_parser *rdf_parser, const unsigned char *string)
 {
-  unsigned char c;
   int len=strlen((const char*)string);
-  int unichar_len;
-  unsigned long unichar;
-  int pos;
-  
-  for(pos=0; (c=*string); string++, len--, pos++) {
+#ifdef RAPTOR_XML_1_1
+  #define XML_ID_XML_VERSION 11
+#else
+  #define XML_ID_XML_VERSION 10
+#endif
 
-    /* It is unicode */
-    
-    unichar_len=raptor_utf8_to_unicode_char(NULL, (const unsigned char *)string, len);
-    if(unichar_len < 0 || unichar_len > len) {
-      raptor_parser_error(rdf_parser, "Bad UTF-8 encoding missing.");
-      return 0;
-    }
-  
-    unichar_len=raptor_utf8_to_unicode_char(&unichar,
-                                            (const unsigned char *)string, len);
-    if(!pos) {
-      /* start of xml:ID name */
-      if(!raptor_unicode_is_namestartchar(unichar))
-        return 0;
-    } else {
-      /* rest of xml:ID name */
-      if(!raptor_unicode_is_namechar(unichar))
-        return 0;
-    }
-
-    unichar_len--; /* since loop does len-- */
-    string += unichar_len; len -= unichar_len;
-  }
-  return 1;
+  return raptor_xml_name_check(string, len, XML_ID_XML_VERSION);
 }
 
 
@@ -242,6 +218,151 @@ raptor_xml_escape_string(const unsigned char *string, size_t len,
   return new_len;
 }
 
+
+/**
+ * raptor_iostream_write_xml_escaped_string - write an XML-escaped version of a string to an iostream
+ * @string: string to XML escape (UTF-8)
+ * @len: length of string
+ * @quote: optional quote character to escape for attribute content, or 0
+ * @iostr: the &raptor_iostream to write to
+ * @error_handler: error handler function
+ * @error_data: error handler data
+ * 
+ * Follows Canonical XML rules on Text Nodes and Attribute Nodes
+ *   http://www.w3.org/TR/xml-c14n#ProcessingModel
+ *
+ * Both:
+ *   Replaces & and < with &amp; and &lt; respectively, preserving other
+ *   characters.
+ * 
+ * Text Nodes:
+ *   > is turned into &gt; #xD is turned into &#xD;
+ *
+ * Attribute Nodes:
+ *   > is generated not &gt.  #x9, #xA and #xD are turned into
+ *   &#x9; &#xA; and &#xD; entities.
+ *
+ * If quote is given it can be either of '\'' or '\"'
+ * which will be turned into &apos; or &quot; respectively.
+ * ASCII NUL ('\0') or any other character will not be escaped.
+ * 
+ * Return value: non 0 on failure
+ **/
+int
+raptor_iostream_write_xml_escaped_string(raptor_iostream* iostr,
+                                         const unsigned char *string,
+                                         size_t len,
+                                         char quote,
+                                         raptor_simple_message_handler error_handler,
+                                         void *error_data)
+{
+  int l;
+  const unsigned char *p;
+
+  if(quote != '\"' && quote != '\'')
+    quote='\0';
+
+  for(l=len, p=string; l; p++, l--) {
+    int unichar_len=1;
+    unsigned long unichar=*p;
+
+    if(*p > 0x7f) {
+      unichar_len=raptor_utf8_to_unicode_char(&unichar, p, l);
+      if(unichar_len < 0 || unichar_len > l) {
+        if(error_handler)
+          error_handler(error_data, "Bad UTF-8 encoding.");
+        return 1;
+      }
+    }
+
+    if(unichar == '&')
+      raptor_iostream_write_counted_string(iostr, "&amp;", 5);
+    else if (unichar == '<')
+      raptor_iostream_write_counted_string(iostr, "&lt;", 4);
+    else if (!quote && unichar == '>')
+      raptor_iostream_write_counted_string(iostr, "&gt;", 4);
+    else if (quote && unichar == (unsigned long)quote) {
+      if(quote == '\'')  
+        raptor_iostream_write_counted_string(iostr, "&apos;", 6);
+      else
+        raptor_iostream_write_counted_string(iostr, "&quot;", 6);
+    } else if (unichar == 0x0d ||
+               (quote && (unichar == 0x09 || unichar == 0x0a))) {
+      /* &#xX; */
+      raptor_iostream_write_counted_string(iostr, "&#x", 3);
+      if(unichar == 0x09)
+        raptor_iostream_write_byte(iostr, '9');
+      else
+        raptor_iostream_write_byte(iostr, 'A'+ ((char)unichar-0x0a));
+      raptor_iostream_write_byte(iostr,  ';');
+    } else
+      raptor_iostream_write_counted_string(iostr, (const char*)p, unichar_len);
+
+    unichar_len--; /* since loop does len-- */
+    p += unichar_len; l -= unichar_len;
+  }
+
+  return 0;
+}
+
+
+/**
+ * raptor_xml_name_check - Check a string is a legal XML name (and legal UTF8)
+ * @string: UTF-8 name string
+ * @length: length of string
+ * @xml_version: XML version
+ * 
+ * xml_version is either 10 (for XML 1.0) or 11 for (XML 1.1). Any
+ * other version fails.
+ *
+ * Return value: Non 0 if the string is a legal XML name
+ **/
+int
+raptor_xml_name_check(const unsigned char *string, size_t length,
+                      int xml_version)
+{
+  int pos;
+
+  if(xml_version != 10 && xml_version != 11)
+    return 0;
+
+  for(pos=0; length > 0; pos++) {
+    unsigned long unichar=0;
+
+    int unichar_len=raptor_utf8_to_unicode_char(&unichar, string, length);
+    if(unichar_len < 0 || unichar_len > (int)length)
+      return 0;
+
+    if(unichar < 0 || unichar > 0x10ffff)
+      return 0;
+  
+    if(!pos) {
+      /* start of name */
+      if(xml_version == 10) {
+        if(!raptor_unicode_is_xml10_namestartchar(unichar))
+          return 0;
+      } else {
+        if(!raptor_unicode_is_xml11_namestartchar(unichar))
+          return 0;
+      }
+    } else {
+      /* rest of name */
+      if(xml_version == 10) {
+        if(!raptor_unicode_is_xml10_namechar(unichar))
+          return 0;
+      } else {
+        if(!raptor_unicode_is_xml11_namechar(unichar))
+          return 0;
+      }
+    }
+
+    string += unichar_len;
+    length -= unichar_len;
+  }
+  return 1;
+}
+
+
 #endif
 
 
