@@ -2,9 +2,9 @@
  *
  * rdfdump.c - Raptor RDF Parser example code 
  *
- * $Id: rdfdump.c,v 1.96 2004/10/20 12:30:56 cmdjb Exp $
+ * $Id: rdfdump.c,v 1.108 2005/05/18 11:13:19 cmdjb Exp $
  *
- * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -177,6 +177,23 @@ rdfdump_warning_handler(void *data, raptor_locator *locator,
   warning_count++;
 }
 
+struct namespace_decl
+{
+  unsigned char *prefix;
+  unsigned char *uri_string;
+};
+
+
+static void
+rdfdump_free_namespace_decl(void* data) {
+  struct namespace_decl* nsd=(struct namespace_decl*)data;
+  if(nsd->prefix)
+    raptor_free_memory(nsd->prefix);
+  if(nsd->uri_string)
+    raptor_free_memory(nsd->uri_string);
+  raptor_free_memory(nsd);
+}
+
 
 int
 main(int argc, char *argv[]) 
@@ -197,8 +214,13 @@ main(int argc, char *argv[])
   raptor_uri *uri;
   char *p;
   char *filename=NULL;
-  raptor_feature feature;
-  int feature_value= -1;
+  raptor_feature parser_feature=(raptor_feature)-1;
+  int parser_feature_value= -1;
+  unsigned char* parser_feature_string_value=NULL;
+  raptor_feature serializer_feature=(raptor_feature)-1;
+  int serializer_feature_value= -1;
+  unsigned char* serializer_feature_string_value=NULL;
+  raptor_sequence *namespace_declarations=NULL;
 
   program=argv[0];
   if((p=strrchr(program, '/')))
@@ -208,7 +230,7 @@ main(int argc, char *argv[])
   argv[0]=program;
 
   raptor_init();
-  
+
   while (!usage && !help)
   {
     int c;
@@ -240,38 +262,99 @@ main(int argc, char *argv[])
           if(!strcmp(optarg, "help")) {
             int i;
             
-            fprintf(stderr, "%s: Valid features for `" HELP_ARG(f, feature) "' are:\n", program);
-            for(i=0; 1; i++) {
+            fprintf(stderr, "%s: Valid parser features are:\n", program);
+            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
               const char *feature_name;
               const char *feature_label;
-              if(raptor_features_enumerate((raptor_feature)i, &feature_name, NULL, &feature_label))
-                break;
-              printf("  %-20s  %s\n", feature_name, feature_label);
+              if(!raptor_features_enumerate((raptor_feature)i, &feature_name, NULL, &feature_label)) {
+                const char *feature_type=(raptor_feature_value_type((raptor_feature)i) == 0) ? "" : " (string)";
+                printf("  %-20s  %s%s\n", feature_name, feature_label, 
+                       feature_type);
+              }
             }
-            fputs("\nFeature values are decimal integers 0 or larger, defaulting to 1 if omitted.\n", stderr);
+            fprintf(stderr, "%s: Valid serializer features are:\n", program);
+            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
+              const char *feature_name;
+              const char *feature_label;
+              if(!raptor_serializer_features_enumerate((raptor_feature)i, &feature_name, NULL, &feature_label)) {
+                const char *feature_type=(raptor_feature_value_type((raptor_feature)i) == 0) ? "" : " (string)";
+                printf("  %-20s  %s%s\n", feature_name, feature_label, 
+                       feature_type);
+              }
+            }
+            fputs("Features are set with `" HELP_ARG(f, feature) " FEATURE=VALUE or `-f FEATURE'\nand take a decimal integer VALUE except where noted, defaulting to 1 if omitted.\n", stderr);
+            fputs("\nA feature of the form xmlns:PREFIX=\"URI\" can be used to declare output\nnamespace prefixes and names for serializing using an XML-style syntax\nEither or both of PREFIX or URI can be omitted such as -f xmlns=\"URI\"\nThis form can be repeated for multiple declarations.\n", stderr);
             exit(0);
+          } else if(!strncmp(optarg, "xmlns", 5)) {
+            struct namespace_decl *nd;
+            nd=(struct namespace_decl *)raptor_alloc_memory(sizeof(struct namespace_decl));
+            if(raptor_new_namespace_parts_from_string((unsigned char*)optarg,
+                                                      &nd->prefix,
+                                                      &nd->uri_string)) {
+              fprintf(stderr, "%s: Bad xmlns syntax in '%s'\n", program, 
+                      optarg);
+              rdfdump_free_namespace_decl(nd);
+              exit(0);
+            }
+
+            if(!namespace_declarations)
+              namespace_declarations=raptor_new_sequence(rdfdump_free_namespace_decl, NULL);
+
+            raptor_sequence_push(namespace_declarations, nd);
           } else {
             int i;
             size_t arg_len=strlen(optarg);
             
-            for(i=0; 1; i++) {
+            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
               const char *feature_name;
               size_t len;
               
               if(raptor_features_enumerate((raptor_feature)i, &feature_name, NULL, NULL))
-                break;
+                continue;
               len=strlen(feature_name);
               if(!strncmp(optarg, feature_name, len)) {
-                feature=(raptor_feature)i;
-                if(len < arg_len && optarg[len] == '=')
-                  feature_value=atoi(&optarg[len+1]);
-                else if(len == arg_len)
-                  feature_value=1;
+                parser_feature=(raptor_feature)i;
+                if(raptor_feature_value_type(parser_feature) == 0) {
+                  if(len < arg_len && optarg[len] == '=')
+                    parser_feature_value=atoi(&optarg[len+1]);
+                  else if(len == arg_len)
+                    parser_feature_value=1;
+                } else {
+                  if(len < arg_len && optarg[len] == '=')
+                    parser_feature_string_value=(unsigned char*)&optarg[len+1];
+                  else if(len == arg_len)
+                    parser_feature_string_value=(unsigned char*)"";
+                }
                 break;
               }
             }
             
-            if(feature_value < 0 ) {
+            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
+              const char *feature_name;
+              size_t len;
+              
+              if(raptor_serializer_features_enumerate((raptor_feature)i, &feature_name, NULL, NULL))
+                continue;
+              len=strlen(feature_name);
+              if(!strncmp(optarg, feature_name, len)) {
+                serializer_feature=(raptor_feature)i;
+                if(raptor_feature_value_type(serializer_feature) == 0) {
+                  if(len < arg_len && optarg[len] == '=')
+                    serializer_feature_value=atoi(&optarg[len+1]);
+                  else if(len == arg_len)
+                    serializer_feature_value=1;
+                } else {
+                  if(len < arg_len && optarg[len] == '=')
+                    serializer_feature_string_value=(unsigned char*)&optarg[len+1];
+                  else if(len == arg_len)
+                    serializer_feature_string_value=(unsigned char*)"";
+                }
+                break;
+              }
+            }
+            
+            if(parser_feature_value < 0 && !parser_feature_string_value &&
+               serializer_feature_value < 0 && !serializer_feature_string_value) {
               fprintf(stderr, "%s: invalid argument `%s' for `" HELP_ARG(f, feature) "'\nTry '%s " HELP_ARG(f, feature) " help' for a list of valid features\n",
                       program, optarg, program);
               usage=1;
@@ -408,25 +491,25 @@ main(int argc, char *argv[])
     puts("Parse RDF content at the source URI into RDF triples.");
     puts("\nMain options:");
     puts(HELP_TEXT("h", "help            ", "Print this help, then exit"));
-    puts(HELP_TEXT("i FORMAT", "input FORMAT ", "Set input format to one of:"));
+    puts(HELP_TEXT("i FORMAT", "input FORMAT ", "Set the input format to one of:"));
     for(i=0; 1; i++) {
       const char *help_name;
       const char *help_label;
       if(raptor_syntaxes_enumerate(i, &help_name, &help_label, NULL, NULL))
         break;
-      printf("    %-12s            %s", help_name, help_label);
+      printf("    %-14s          %s", help_name, help_label);
       if(!i)
         puts(" (default)");
       else
         putchar('\n');
     }
-    puts(HELP_TEXT("o FORMAT", "output FORMAT", "Set output format to one of:"));
+    puts(HELP_TEXT("o FORMAT", "output FORMAT", "Set the output format to one of:"));
     for(i=0; 1; i++) {
       const char *help_name;
       const char *help_label;
       if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
         break;
-      printf("    %-12s            %s", help_name, help_label);
+      printf("    %-14s          %s", help_name, help_label);
       if(!i)
         puts(" (default)");
       else
@@ -434,17 +517,20 @@ main(int argc, char *argv[])
     }
     puts(HELP_TEXT("m MODE", "mode MODE  ", "Set parser mode - 'lax' (default) or 'strict'"));
     puts("\nAdditional options:");
-    puts(HELP_TEXT("c", "count           ", "Count triples - no output"));
+    puts(HELP_TEXT("c", "count           ", "Count triples only - do not print them."));
     puts(HELP_TEXT("e", "ignore-errors   ", "Ignore error messages"));
-    puts(HELP_TEXT("f FEATURE(=VALUE)", "feature FEATURE(=VALUE)", HELP_PAD "Set parser feature - use `help' for a list"));
-    puts(HELP_TEXT("g", "guess           ", "Guess input syntax"));
+    puts(HELP_TEXT("f FEATURE(=VALUE)", "feature FEATURE(=VALUE)", HELP_PAD "Set parser or serializer features" HELP_PAD "Use `-f help' for a list of valid features"));
+    puts(HELP_TEXT("g", "guess           ", "Guess the input syntax"));
     puts(HELP_TEXT("q", "quiet           ", "No extra information messages"));
     puts(HELP_TEXT("r", "replace-newlines", "Replace newlines with spaces in literals"));
     puts(HELP_TEXT("s", "scan            ", "Scan for <rdf:RDF> element in source"));
     puts(HELP_TEXT("w", "ignore-warnings ", "Ignore warning messages"));
     puts(HELP_TEXT("v", "version         ", "Print the Raptor version"));
-    puts("\nReport bugs to <redland-dev@lists.librdf.org>.");
+    puts("\nReport bugs to http://bugs.librdf.org/");
     puts("Raptor home page: http://librdf.org/raptor/");
+
+    raptor_finish();
+
     exit(0);
   }
 
@@ -524,8 +610,14 @@ main(int argc, char *argv[])
   if(scanning)
     raptor_set_feature(rdf_parser, RAPTOR_FEATURE_SCANNING, 1);
 
-  if(feature_value >= 0)
-    raptor_set_feature(rdf_parser, feature, feature_value);
+  if(parser_feature_value >= 0)
+    raptor_set_feature(rdf_parser, 
+                       parser_feature, parser_feature_value);
+  if(parser_feature_string_value)
+    raptor_parser_set_feature_string(rdf_parser, 
+                                     parser_feature,
+                                     parser_feature_string_value);
+
 
   if(!quiet) {
     if (filename) {
@@ -554,7 +646,33 @@ main(int argc, char *argv[])
               serializer_syntax_name);
       return(1);
     }
+
+    if(namespace_declarations) {
+      int i;
+      for(i=0; i< raptor_sequence_size(namespace_declarations); i++) {
+        struct namespace_decl *nd=(struct namespace_decl *)raptor_sequence_get_at(namespace_declarations, i);
+        raptor_uri *ns_uri=NULL;
+        if(nd->uri_string)
+          ns_uri=raptor_new_uri(nd->uri_string);
+        
+        raptor_serialize_set_namespace(serializer, ns_uri, nd->prefix);
+        if(ns_uri)
+          raptor_free_uri(ns_uri);
+      }
+      raptor_free_sequence(namespace_declarations);
+      namespace_declarations=NULL;
+    }
+    
+    if(serializer_feature_value >= 0)
+      raptor_serializer_set_feature(serializer, 
+                                    serializer_feature, serializer_feature_value);
+    if(serializer_feature_string_value)
+      raptor_serializer_set_feature_string(serializer, 
+                                           serializer_feature,
+                                           serializer_feature_string_value);
+
     raptor_serialize_start_to_file_handle(serializer, base_uri, stdout);
+
   }
   
 
@@ -592,6 +710,9 @@ main(int argc, char *argv[])
   if(free_uri_string)
     raptor_free_memory(uri_string);
 
+  if(namespace_declarations)
+    raptor_free_sequence(namespace_declarations);
+
   raptor_finish();
 
   if(error_count && !ignore_errors)
