@@ -2,7 +2,7 @@
  *
  * rasqal.h - Rasqal RDF Query library interfaces and definition
  *
- * $Id: rasqal.h,v 1.81 2005/02/03 15:04:23 cmdjb Exp $
+ * $Id: rasqal.h,v 1.112 2005/06/08 16:20:24 cmdjb Exp $
  *
  * Copyright (C) 2003-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -26,8 +26,8 @@
 
 
 
-#ifndef RDQL_H
-#define RDQL_H
+#ifndef RASQAL_H
+#define RASQAL_H
 
 
 #ifdef __cplusplus
@@ -45,6 +45,21 @@ extern "C" {
 #  define RASQAL_API
 #endif
 
+/* Use gcc 3.1+ feature to allow marking of deprecated API calls.
+ * This gives a warning during compiling.
+ */
+#if ( __GNUC__ == 3 && __GNUC_MINOR__ > 0 ) || __GNUC__ > 3
+#ifdef __APPLE_CC__
+/* OSX gcc cpp-precomp is broken */
+#define RASQAL_DEPRECATED
+#else
+#define RASQAL_DEPRECATED __attribute__((deprecated))
+#endif
+#else
+#define RASQAL_DEPRECATED
+#endif
+
+
 #ifndef LIBRDF_OBJC_FRAMEWORK
 #include <raptor.h>
 #else
@@ -52,19 +67,20 @@ extern "C" {
 #endif
 
 /* Public statics */
-extern const char * const rasqal_short_copyright_string;
-extern const char * const rasqal_copyright_string;
-extern const char * const rasqal_version_string;
-extern const unsigned int rasqal_version_major;
-extern const unsigned int rasqal_version_minor;
-extern const unsigned int rasqal_version_release;
-extern const unsigned int rasqal_version_decimal;
+RASQAL_API extern const char * const rasqal_short_copyright_string;
+RASQAL_API extern const char * const rasqal_copyright_string;
+RASQAL_API extern const char * const rasqal_version_string;
+RASQAL_API extern const unsigned int rasqal_version_major;
+RASQAL_API extern const unsigned int rasqal_version_minor;
+RASQAL_API extern const unsigned int rasqal_version_release;
+RASQAL_API extern const unsigned int rasqal_version_decimal;
 
 
 /* Public structure */
 typedef struct rasqal_query_s rasqal_query;
 typedef struct rasqal_query_results_s rasqal_query_results;
 typedef struct rasqal_literal_s rasqal_literal;
+typedef struct rasqal_graph_pattern_s rasqal_graph_pattern;
 
 typedef enum {
   RASQAL_FEATURE_LAST
@@ -78,14 +94,36 @@ typedef struct {
 } rasqal_prefix;
 
 
+typedef enum {
+  RASQAL_VARIABLE_TYPE_UNKNOWN   = 0,
+  RASQAL_VARIABLE_TYPE_NORMAL    = 1,
+  RASQAL_VARIABLE_TYPE_ANONYMOUS = 2
+} rasqal_variable_type;
+
+
 /* variable binding */
 typedef struct {
   const unsigned char *name;
   struct rasqal_literal_s *value;
   int offset;   /* offset in the rasqal_query variables array */
+  rasqal_variable_type type;     /* variable type */
 } rasqal_variable;
 
 
+typedef enum {
+  RASQAL_DATA_GRAPH_NONE  = 0,
+  RASQAL_DATA_GRAPH_NAMED = 1,
+  RASQAL_DATA_GRAPH_BACKGROUND = 2,
+} rasqal_data_graph_flags;
+
+
+typedef struct {
+  raptor_uri* uri;
+  raptor_uri* name_uri;
+  int flags;
+} rasqal_data_graph;
+
+
 typedef enum {
   RASQAL_LITERAL_UNKNOWN,
   RASQAL_LITERAL_URI,
@@ -151,7 +189,6 @@ typedef enum {
   RASQAL_EXPR_TILDE,
   RASQAL_EXPR_BANG,
   RASQAL_EXPR_LITERAL,
-  RASQAL_EXPR_PATTERN,
   RASQAL_EXPR_FUNCTION,
   RASQAL_EXPR_BOUND,
   RASQAL_EXPR_STR,
@@ -160,7 +197,10 @@ typedef enum {
   RASQAL_EXPR_ISURI,
   RASQAL_EXPR_ISBLANK,
   RASQAL_EXPR_ISLITERAL,
-  RASQAL_EXPR_LAST= RASQAL_EXPR_ISLITERAL
+  RASQAL_EXPR_CAST,
+  RASQAL_EXPR_ORDER_COND_ASC,
+  RASQAL_EXPR_ORDER_COND_DESC,
+  RASQAL_EXPR_LAST= RASQAL_EXPR_ORDER_COND_DESC
 } rasqal_op;
 
 
@@ -177,7 +217,7 @@ struct rasqal_expression_s {
   rasqal_variable* variable;
   unsigned char *value; /* UTF-8 value */
 
-  /* for extension function qname(args...) */
+  /* for extension function qname(args...) and cast-to-uri */
   raptor_uri* name;
   raptor_sequence* args;
 };
@@ -186,10 +226,13 @@ typedef struct rasqal_expression_s rasqal_expression;
 
 /* Extra flags for triples */
 typedef enum {
-  /* true when all of subject, predicate, object are given */
+
+  /* Not used - was only used internally in the execution engine */
   RASQAL_TRIPLE_FLAGS_EXACT=1,
-  /* true when the triple is an optional match */
+
+  /* Not used - this is now a property of a graph pattern */
   RASQAL_TRIPLE_FLAGS_OPTIONAL=2,
+
   RASQAL_TRIPLE_FLAGS_LAST=RASQAL_TRIPLE_FLAGS_OPTIONAL
 } rasqal_triple_flags;
 
@@ -204,6 +247,47 @@ typedef struct {
 } rasqal_triple ;
 
 
+/* Flags for graph patterns  */
+typedef enum {
+  /* true when the graph pattern is an optional match */
+  RASQAL_PATTERN_FLAGS_OPTIONAL=1,
+
+  RASQAL_PATTERN_FLAGS_LAST=RASQAL_PATTERN_FLAGS_OPTIONAL
+} rasqal_pattern_flags;
+
+
+typedef unsigned char* (*rasqal_generate_bnodeid_handler)(rasqal_query* query, void *user_data, unsigned char *user_bnodeid);
+
+
+/* Query verbs  */
+typedef enum {
+  RASQAL_QUERY_VERB_UNKNOWN   = 0,
+  RASQAL_QUERY_VERB_SELECT    = 1,
+  RASQAL_QUERY_VERB_CONSTRUCT = 2,
+  RASQAL_QUERY_VERB_DESCRIBE  = 3,
+  RASQAL_QUERY_VERB_ASK       = 4,
+
+  RASQAL_QUERY_VERB_LAST=RASQAL_QUERY_VERB_ASK
+} rasqal_query_verb;
+
+
+/* Graph pattern operators  */
+typedef enum {
+  RASQAL_GRAPH_PATTERN_OPERATOR_UNKNOWN   = 0,
+  /* Basic - just triple patterns and constraints */
+  RASQAL_GRAPH_PATTERN_OPERATOR_BASIC     = 1,
+  /* Optional - set of graph patterns (ANDed) and constraints */
+  RASQAL_GRAPH_PATTERN_OPERATOR_OPTIONAL  = 2,
+  /* Union - set of graph patterns (UNIONed) and constraints */
+  RASQAL_GRAPH_PATTERN_OPERATOR_UNION     = 3,
+  /* Group - set of graph patterns (ANDed) and constraints */
+  RASQAL_GRAPH_PATTERN_OPERATOR_GROUP     = 4,
+  /* Graph - a graph term + a graph pattern and constraints */
+  RASQAL_GRAPH_PATTERN_OPERATOR_GRAPH     = 5,
+
+  RASQAL_GRAPH_PATTERN_OPERATOR_LAST=RASQAL_GRAPH_PATTERN_OPERATOR_GRAPH
+} rasqal_graph_pattern_operator;
+
 
 /* RASQAL API */
 
@@ -231,26 +315,56 @@ RASQAL_API void rasqal_query_set_fatal_error_handler(rasqal_query* query, void *
 RASQAL_API void rasqal_query_set_error_handler(rasqal_query* query, void *user_data, raptor_message_handler handler);
 RASQAL_API void rasqal_query_set_warning_handler(rasqal_query* query, void *user_data, raptor_message_handler handler);
 RASQAL_API void rasqal_query_set_feature(rasqal_query *query, rasqal_feature feature, int value);
+RASQAL_API void rasqal_query_set_default_generate_bnodeid_parameters(rasqal_query* rdf_query, char *prefix, int base);
+RASQAL_API void rasqal_query_set_generate_bnodeid_handler(rasqal_query* query, void *user_data, rasqal_generate_bnodeid_handler handler);
+
+RASQAL_API rasqal_query_verb rasqal_query_get_verb(rasqal_query *query);
+RASQAL_API int rasqal_query_get_wildcard(rasqal_query *query);
+RASQAL_API int rasqal_query_get_distinct(rasqal_query *query);
+RASQAL_API void rasqal_query_set_distinct(rasqal_query *query, int is_distinct);
+RASQAL_API int rasqal_query_get_limit(rasqal_query *query);
+RASQAL_API void rasqal_query_set_limit(rasqal_query *query, int limit);
+RASQAL_API int rasqal_query_get_offset(rasqal_query *query);
+RASQAL_API void rasqal_query_set_offset(rasqal_query *query, int limit);
+
+RASQAL_API int rasqal_query_add_data_graph(rasqal_query* query, raptor_uri* uri, raptor_uri* name_uri, int flags);
+RASQAL_API raptor_sequence* rasqal_query_get_data_graph_sequence(rasqal_query* query);
+RASQAL_API rasqal_data_graph* rasqal_query_get_data_graph(rasqal_query* query, int idx);
 
-RASQAL_API void rasqal_query_add_source(rasqal_query* query, raptor_uri* uri);
-RASQAL_API raptor_sequence* rasqal_query_get_source_sequence(rasqal_query* query);
-RASQAL_API raptor_uri* rasqal_query_get_source(rasqal_query* query, int idx);
 RASQAL_API void rasqal_query_add_variable(rasqal_query* query, rasqal_variable* var);
-RASQAL_API raptor_sequence* rasqal_query_get_variable_sequence(rasqal_query* query);
+RASQAL_API raptor_sequence* rasqal_query_get_bound_variable_sequence(rasqal_query* query);
+RASQAL_API raptor_sequence* rasqal_query_get_all_variable_sequence(rasqal_query* query);
 RASQAL_API rasqal_variable* rasqal_query_get_variable(rasqal_query* query, int idx);
 RASQAL_API int rasqal_query_has_variable(rasqal_query* query, const unsigned char *name);
 RASQAL_API int rasqal_query_set_variable(rasqal_query* query, const unsigned char *name, rasqal_literal* value);
-RASQAL_API void rasqal_query_add_triple(rasqal_query* query, rasqal_triple* triple);
 RASQAL_API raptor_sequence* rasqal_query_get_triple_sequence(rasqal_query* query);
 RASQAL_API rasqal_triple* rasqal_query_get_triple(rasqal_query* query, int idx);
-RASQAL_API void rasqal_query_add_constraint(rasqal_query* query, rasqal_expression* expr);
-RASQAL_API raptor_sequence* rasqal_query_get_constraint_sequence(rasqal_query* query);
-RASQAL_API rasqal_expression* rasqal_query_get_constraint(rasqal_query* query, int idx);
 RASQAL_API void rasqal_query_add_prefix(rasqal_query* query, rasqal_prefix* prefix);
 RASQAL_API raptor_sequence* rasqal_query_get_prefix_sequence(rasqal_query* query);
 RASQAL_API rasqal_prefix* rasqal_query_get_prefix(rasqal_query* query, int idx);
+RASQAL_API raptor_sequence* rasqal_query_get_order_conditions_sequence(rasqal_query* query);
+RASQAL_API rasqal_expression* rasqal_query_get_order_condition(rasqal_query* query, int idx);
+
+/* graph patterns */
+RASQAL_API rasqal_graph_pattern* rasqal_query_get_query_graph_pattern(rasqal_query* query);
+RASQAL_API raptor_sequence* rasqal_query_get_graph_pattern_sequence(rasqal_query* query);
+RASQAL_API rasqal_graph_pattern* rasqal_query_get_graph_pattern(rasqal_query* query, int idx);
+RASQAL_API void rasqal_graph_pattern_add_sub_graph_pattern(rasqal_graph_pattern* graph_pattern, rasqal_graph_pattern* sub_graph_pattern);
+RASQAL_API rasqal_triple* rasqal_graph_pattern_get_triple(rasqal_graph_pattern* graph_pattern, int idx);
+RASQAL_API raptor_sequence* rasqal_graph_pattern_get_sub_graph_pattern_sequence(rasqal_graph_pattern* graph_pattern);
+RASQAL_API rasqal_graph_pattern* rasqal_graph_pattern_get_sub_graph_pattern(rasqal_graph_pattern* graph_pattern, int idx);
+RASQAL_API RASQAL_DEPRECATED int rasqal_graph_pattern_get_flags(rasqal_graph_pattern* graph_pattern);
+RASQAL_API rasqal_graph_pattern_operator rasqal_graph_pattern_get_operator(rasqal_graph_pattern* graph_pattern);
+RASQAL_API const char* rasqal_graph_pattern_operator_as_string(rasqal_graph_pattern_operator verb);
+RASQAL_API void rasqal_graph_pattern_print(rasqal_graph_pattern* gp, FILE* fh);
+RASQAL_API int rasqal_graph_pattern_add_constraint(rasqal_graph_pattern* gp, rasqal_expression* expr);
+RASQAL_API raptor_sequence* rasqal_graph_pattern_get_constraint_sequence(rasqal_graph_pattern* gp);
+RASQAL_API rasqal_expression* rasqal_graph_pattern_get_constraint(rasqal_graph_pattern* gp, int idx);
+RASQAL_API raptor_sequence* rasqal_query_get_construct_triples_sequence(rasqal_query* query);
+RASQAL_API rasqal_triple* rasqal_query_get_construct_triple(rasqal_query* query, int idx);
 
 /* Utility methods */
+RASQAL_API const char* rasqal_query_verb_as_string(rasqal_query_verb verb);
 RASQAL_API void rasqal_query_print(rasqal_query* query, FILE *stream);
 
 /* Query */
@@ -283,7 +397,13 @@ RASQAL_API int rasqal_query_results_is_graph(rasqal_query_results *query_results
 RASQAL_API raptor_statement* rasqal_query_results_get_triple(rasqal_query_results *query_results);
 RASQAL_API int rasqal_query_results_next_triple(rasqal_query_results *query_results);
 
-RAPTOR_API int rasqal_query_results_write(raptor_iostream *iostr, rasqal_query_results *results, raptor_uri *format_uri, raptor_uri *base_uri);
+RASQAL_API int rasqal_query_results_write(raptor_iostream *iostr, rasqal_query_results *results, raptor_uri *format_uri, raptor_uri *base_uri);
+
+/* Data graph class */
+RASQAL_API rasqal_data_graph* rasqal_new_data_graph(raptor_uri* uri, raptor_uri* name_uri, int flags);
+RASQAL_API void rasqal_free_data_graph(rasqal_data_graph* dg);
+RASQAL_API void rasqal_data_graph_print(rasqal_data_graph* dg, FILE* fh);
+
 
 /* Expression class */
 RASQAL_API rasqal_expression* rasqal_new_1op_expression(rasqal_op op, rasqal_expression* arg);
@@ -292,6 +412,7 @@ RASQAL_API rasqal_expression* rasqal_new_string_op_expression(rasqal_op op, rasq
 RASQAL_API rasqal_expression* rasqal_new_literal_expression(rasqal_literal* literal);
 RASQAL_API rasqal_expression* rasqal_new_variable_expression(rasqal_variable *variable);
 RASQAL_API rasqal_expression* rasqal_new_function_expression(raptor_uri* name, raptor_sequence* args);
+RASQAL_API rasqal_expression* rasqal_new_cast_expression(raptor_uri* name, rasqal_expression *value);
 
 RASQAL_API void rasqal_free_expression(rasqal_expression* expr);
 RASQAL_API void rasqal_expression_print_op(rasqal_expression* expr, FILE* fh);
@@ -333,15 +454,19 @@ RASQAL_API void rasqal_free_triple(rasqal_triple* t);
 RASQAL_API void rasqal_triple_print(rasqal_triple* t, FILE* fh);
 RASQAL_API void rasqal_triple_set_origin(rasqal_triple* t, rasqal_literal *l);
 RASQAL_API rasqal_literal* rasqal_triple_get_origin(rasqal_triple* t);
-RASQAL_API void rasqal_triple_set_flags(rasqal_triple* t, unsigned int flags);
-RASQAL_API unsigned int rasqal_triple_get_flags(rasqal_triple* t);
 
 /* Variable class */
-RASQAL_API rasqal_variable* rasqal_new_variable(rasqal_query* query, const unsigned char *name, rasqal_literal *value);
+RASQAL_API rasqal_variable* rasqal_new_variable_typed(rasqal_query* rq, rasqal_variable_type type, unsigned char *name, rasqal_literal *value);
+RASQAL_API rasqal_variable* rasqal_new_variable(rasqal_query* query, unsigned char *name, rasqal_literal *value);
 RASQAL_API void rasqal_free_variable(rasqal_variable* variable);
 RASQAL_API void rasqal_variable_print(rasqal_variable* t, FILE* fh);
 RASQAL_API void rasqal_variable_set_value(rasqal_variable* v, rasqal_literal *l);
 
+/* memory functions */
+RASQAL_API void rasqal_free_memory(void *ptr);
+RASQAL_API void* rasqal_alloc_memory(size_t size);
+RASQAL_API void* rasqal_calloc_memory(size_t nmemb, size_t size);
+
 /* rasqal_engine.c */
 
 typedef enum {
@@ -383,6 +508,9 @@ typedef struct
 
   /* parts of the triple*/
   rasqal_triple_parts parts;
+
+  /* non-0 if all parts of the triple are given */
+  int is_exact;
 } rasqal_triple_meta;
 
 
@@ -393,7 +521,7 @@ struct rasqal_triples_source_s {
   void *user_data;
 
   /* the triples_source_factory initialises these method */
-  rasqal_triples_match* (*new_triples_match)(struct rasqal_triples_source_s* rts, void *user_data, rasqal_triple_meta *m, rasqal_triple *t);
+  int (*init_triples_match)(rasqal_triples_match* rtm, struct rasqal_triples_source_s* rts, void *user_data, rasqal_triple_meta *m, rasqal_triple *t);
 
   int (*triple_present)(struct rasqal_triples_source_s* rts, void *user_data, rasqal_triple *t);
 
@@ -405,6 +533,11 @@ typedef struct rasqal_triples_source_s rasqal_triples_source;
 typedef struct {
   void *user_data; /* user data for triples_source_factory */
   size_t user_data_size; /* size of user data for new_triples_source */
+
+  /**
+   * create a new triples source - returns non-zero on failure
+   * < 0 is a 'no rdf data error', > 0 is an unspecified error
+   */
   int (*new_triples_source)(rasqal_query *query, void *factory_user_data, void *user_data, rasqal_triples_source* rts);
 } rasqal_triples_source_factory;
   
