@@ -2,7 +2,7 @@
  *
  * rasqal_general.c - Rasqal library startup, shutdown and factories
  *
- * $Id: rasqal_general.c,v 1.35 2005/01/21 06:20:03 cmdjb Exp $
+ * $Id: rasqal_general.c,v 1.42 2005/05/18 14:11:44 cmdjb Exp $
  *
  * Copyright (C) 2004-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -87,16 +87,18 @@ rasqal_init(void)
 
   raptor_init();
 
-  /* last one declared is the default - RDQL */
+  rasqal_uri_init();
 
-#ifdef RASQAL_QUERY_SPARQL  
-  rasqal_init_query_engine_sparql();
-#endif
+  /* last one declared is the default - RDQL */
 
 #ifdef RASQAL_QUERY_RDQL
   rasqal_init_query_engine_rdql();
 #endif
 
+#ifdef RASQAL_QUERY_SPARQL  
+  rasqal_init_query_engine_sparql();
+#endif
+
 #ifdef RAPTOR_TRIPLES_SOURCE_RAPTOR
   rasqal_raptor_init();
 #endif
@@ -123,13 +125,15 @@ rasqal_finish(void)
   rasqal_finishing=1;
 
   rasqal_delete_query_engine_factories();
-#ifdef RAPTOR_TRIPLES_SOURCE_RAPTOR
-  raptor_finish();
-#endif
+
 #ifdef RAPTOR_TRIPLES_SOURCE_REDLAND
   rasqal_redland_finish();
 #endif
 
+  rasqal_uri_finish();
+
+  raptor_finish();
+
   rasqal_initialising=0;
   rasqal_initialised=0;
   rasqal_finishing=0;
@@ -153,12 +157,12 @@ rasqal_delete_query_engine_factories(void)
     if(factory->finish_factory)
       factory->finish_factory(factory);
 
-    RASQAL_FREE(rasqal_query_engine_factory, factory->name);
-    RASQAL_FREE(rasqal_query_engine_factory, factory->label);
+    RASQAL_FREE(rasqal_query_engine_factory, (void*)factory->name);
+    RASQAL_FREE(rasqal_query_engine_factory, (void*)factory->label);
     if(factory->alias)
-      RASQAL_FREE(rasqal_query_engine_factory, factory->alias);
+      RASQAL_FREE(rasqal_query_engine_factory, (void*)factory->alias);
     if(factory->uri_string)
-      RASQAL_FREE(rasqal_query_engine_factory, factory->uri_string);
+      RASQAL_FREE(rasqal_query_engine_factory, (void*)factory->uri_string);
 
     RASQAL_FREE(rasqal_query_engine_factory, factory);
   }
@@ -624,6 +628,313 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
 }
 
 
+raptor_uri* rasqal_xsd_namespace_uri=NULL;
+
+raptor_uri* rasqal_xsd_integer_uri=NULL;
+raptor_uri* rasqal_xsd_double_uri=NULL;
+raptor_uri* rasqal_xsd_boolean_uri=NULL;
+
+raptor_uri* rasqal_rdf_namespace_uri=NULL;
+
+raptor_uri* rasqal_rdf_first_uri=NULL;
+raptor_uri* rasqal_rdf_rest_uri=NULL;
+raptor_uri* rasqal_rdf_nil_uri=NULL;
+
+
+void
+rasqal_uri_init() 
+{
+  rasqal_xsd_namespace_uri=raptor_new_uri(raptor_xmlschema_datatypes_namespace_uri);
+  
+  rasqal_xsd_integer_uri=raptor_new_uri_from_uri_local_name(rasqal_xsd_namespace_uri, (const unsigned char*)"integer");
+  rasqal_xsd_double_uri=raptor_new_uri_from_uri_local_name(rasqal_xsd_namespace_uri, (const unsigned char*)"double");
+  rasqal_xsd_boolean_uri=raptor_new_uri_from_uri_local_name(rasqal_xsd_namespace_uri, (const unsigned char*)"boolean");
+
+  rasqal_rdf_namespace_uri=raptor_new_uri(raptor_rdf_namespace_uri);
+
+  rasqal_rdf_first_uri=raptor_new_uri_from_uri_local_name(rasqal_rdf_namespace_uri, (const unsigned char*)"first");
+  rasqal_rdf_rest_uri=raptor_new_uri_from_uri_local_name(rasqal_rdf_namespace_uri, (const unsigned char*)"rest");
+  rasqal_rdf_nil_uri=raptor_new_uri_from_uri_local_name(rasqal_rdf_namespace_uri, (const unsigned char*)"nil");
+  
+}
+
+
+void
+rasqal_uri_finish() 
+{
+  if(rasqal_xsd_integer_uri)
+    raptor_free_uri(rasqal_xsd_integer_uri);
+  if(rasqal_xsd_double_uri)
+    raptor_free_uri(rasqal_xsd_double_uri);
+  if(rasqal_xsd_boolean_uri)
+    raptor_free_uri(rasqal_xsd_boolean_uri);
+  if(rasqal_xsd_namespace_uri)
+    raptor_free_uri(rasqal_xsd_namespace_uri);
+
+  if(rasqal_rdf_first_uri)
+    raptor_free_uri(rasqal_rdf_first_uri);
+  if(rasqal_rdf_rest_uri)
+    raptor_free_uri(rasqal_rdf_rest_uri);
+  if(rasqal_rdf_nil_uri)
+    raptor_free_uri(rasqal_rdf_nil_uri);
+  if(rasqal_rdf_namespace_uri)
+    raptor_free_uri(rasqal_rdf_namespace_uri);
+
+}
+
+
+
+/**
+ * rasqal_query_set_default_generate_bnodeid_parameters - Set default bnodeid generation parameters
+ * @rdf_query: &rasqal_parse object
+ * @prefix: prefix string
+ * @base: integer base identifier
+ *
+ * Sets the parameters for the default algorithm used to generate
+ * blank node IDs.  The default algorithm uses both @prefix and @base
+ * to generate a new identifier.  The exact identifier generated is
+ * not guaranteed to be a strict concatenation of @prefix and @base
+ * but will use both parts.
+ *
+ * For finer control of the generated identifiers, use
+ * &rasqal_set_default_generate_bnodeid_handler.
+ *
+ * If prefix is NULL, the default prefix is used (currently "bnodeid")
+ * If base is less than 1, it is initialised to 1.
+ * 
+ **/
+void
+rasqal_query_set_default_generate_bnodeid_parameters(rasqal_query* rdf_query, 
+                                                     char *prefix, int base)
+{
+  char *prefix_copy=NULL;
+  size_t length=0;
+
+  if(--base<0)
+    base=0;
+
+  if(prefix) {
+    length=strlen(prefix);
+    
+    prefix_copy=(char*)RASQAL_MALLOC(cstring, length+1);
+    if(!prefix_copy)
+      return;
+    strcpy(prefix_copy, prefix);
+  }
+  
+  if(rdf_query->default_generate_bnodeid_handler_prefix)
+    RASQAL_FREE(cstring, rdf_query->default_generate_bnodeid_handler_prefix);
+
+  rdf_query->default_generate_bnodeid_handler_prefix=prefix_copy;
+  rdf_query->default_generate_bnodeid_handler_prefix_length=length;
+  rdf_query->default_generate_bnodeid_handler_base=base;
+}
+
+
+/**
+ * rasqal_query_set_generate_bnodeid_handler - Set the generate blank node ID handler function for the query
+ * @query: &rasqal_query query object
+ * @user_data: user data pointer for callback
+ * @handler: generate blank ID callback function
+ *
+ * Sets the function to generate blank node IDs for the query.
+ * The handler is called with a pointer to the rasqal_query, the
+ * &user_data pointer and a user_bnodeid which is the value of
+ * a user-provided blank node identifier (may be NULL).
+ * It can either be returned directly as the generated value when present or
+ * modified.  The passed in value must be free()d if it is not used.
+ *
+ * If handler is NULL, the default method is used
+ * 
+ **/
+void
+rasqal_query_set_generate_bnodeid_handler(rasqal_query* query,
+                                          void *user_data,
+                                          rasqal_generate_bnodeid_handler handler)
+{
+  query->generate_bnodeid_handler_user_data=user_data;
+  query->generate_bnodeid_handler=handler;
+}
+
+
+static unsigned char*
+rasqal_default_generate_bnodeid_handler(void *user_data,
+                                        unsigned char *user_bnodeid) 
+{
+  rasqal_query *rdf_query=(rasqal_query *)user_data;
+  int id;
+  unsigned char *buffer;
+  int length;
+  int tmpid;
+
+  if(user_bnodeid)
+    return user_bnodeid;
+
+  id=++rdf_query->default_generate_bnodeid_handler_base;
+
+  tmpid=id;
+  length=2; /* min length 1 + \0 */
+  while(tmpid/=10)
+    length++;
+
+  if(rdf_query->default_generate_bnodeid_handler_prefix)
+    length += rdf_query->default_generate_bnodeid_handler_prefix_length;
+  else
+    length += 7; /* bnodeid */
+  
+  buffer=(unsigned char*)RASQAL_MALLOC(cstring, length);
+  if(!buffer)
+    return NULL;
+  if(rdf_query->default_generate_bnodeid_handler_prefix) {
+    strncpy((char*)buffer, rdf_query->default_generate_bnodeid_handler_prefix,
+            rdf_query->default_generate_bnodeid_handler_prefix_length);
+    sprintf((char*)buffer + rdf_query->default_generate_bnodeid_handler_prefix_length,
+            "%d", id);
+  } else 
+    sprintf((char*)buffer, "bnodeid%d", id);
+
+  return buffer;
+}
+
+
+/*
+ * rasqal_query_generate_bnodeid - Default generate id - internal
+ */
+unsigned char*
+rasqal_query_generate_bnodeid(rasqal_query* rdf_query,
+                              unsigned char *user_bnodeid)
+{
+  if(rdf_query->generate_bnodeid_handler)
+    return rdf_query->generate_bnodeid_handler(rdf_query, 
+                                               rdf_query->generate_bnodeid_handler_user_data, user_bnodeid);
+  else
+    return rasqal_default_generate_bnodeid_handler(rdf_query, user_bnodeid);
+}
+
+
+
+
+/**
+ * rasqal_free_memory - Free memory allocated inside rasqal.
+ * @ptr: memory pointer
+ * 
+ * Some systems require memory allocated in a library to
+ * be deallocated in that library.  This function allows
+ * memory allocated by rasqal to be freed.
+ *
+ **/
+void
+rasqal_free_memory(void *ptr)
+{
+  RASQAL_FREE(void, ptr);
+}
+
+
+/**
+ * rasqal_alloc_memory - Allocate memory inside rasqal.
+ * @size: size of memory to allocate
+ * 
+ * Some systems require memory allocated in a library to
+ * be deallocated in that library.  This function allows
+ * memory to be allocated inside the rasqal shared library
+ * that can be freed inside rasqal either internally or via
+ * rasqal_free_memory.
+ *
+ * Return value: the address of the allocated memory or NULL on failure
+ *
+ **/
+void*
+rasqal_alloc_memory(size_t size)
+{
+  return RASQAL_MALLOC(void, size);
+}
+
+
+/**
+ * rasqal_calloc_memory - Allocate zeroed array of items inside rasqal.
+ * @nmemb: number of members
+ * @size: size of item
+ * 
+ * Some systems require memory allocated in a library to
+ * be deallocated in that library.  This function allows
+ * memory to be allocated inside the rasqal shared library
+ * that can be freed inside rasqal either internally or via
+ * rasqal_free_memory.
+ *
+ * Return value: the address of the allocated memory or NULL on failure
+ *
+ **/
+void*
+rasqal_calloc_memory(size_t nmemb, size_t size)
+{
+  return RASQAL_CALLOC(void, nmemb, size);
+}
+
+
+#if defined (RASQAL_DEBUG) && defined(RASQAL_MEMORY_SIGN)
+void*
+rasqal_sign_malloc(size_t size)
+{
+  int *p;
+  
+  size += sizeof(int);
+  
+  p=(int*)malloc(size);
+  *p++ = RASQAL_SIGN_KEY;
+  return p;
+}
+
+void*
+rasqal_sign_calloc(size_t nmemb, size_t size)
+{
+  int *p;
+  
+  /* turn into bytes */
+  size = nmemb*size + sizeof(int);
+  
+  p=(int*)calloc(1, size);
+  *p++ = RASQAL_SIGN_KEY;
+  return p;
+}
+
+void*
+rasqal_sign_realloc(void *ptr, size_t size)
+{
+  int *p;
+
+  if(!ptr)
+    return rasqal_sign_malloc(size);
+  
+  p=(int*)ptr;
+  p--;
+
+  if(*p != RASQAL_SIGN_KEY)
+    RASQAL_FATAL3("memory signature %08X != %08X", *p, RASQAL_SIGN_KEY);
+
+  size += sizeof(int);
+  
+  p=(int*)realloc(p, size);
+  *p++= RASQAL_SIGN_KEY;
+  return p;
+}
+
+void
+rasqal_sign_free(void *ptr)
+{
+  int *p;
+
+  if(!ptr)
+    return;
+  
+  p=(int*)ptr;
+  p--;
+
+  if(*p != RASQAL_SIGN_KEY)
+    RASQAL_FATAL3("memory signature %08X != %08X", *p, RASQAL_SIGN_KEY);
+
+  free(p);
+}
+#endif
 
 
 #if defined (RASQAL_DEBUG) && defined(HAVE_DMALLOC_H) && defined(RASQAL_MEMORY_DEBUG_DMALLOC)
