@@ -2,7 +2,7 @@
  *
  * rasqal_internal.h - Rasqal RDF Query library internals
  *
- * $Id: rasqal_internal.h,v 1.74 2005/01/31 21:51:21 cmdjb Exp $
+ * $Id: rasqal_internal.h,v 1.99 2005/06/08 16:33:18 cmdjb Exp $
  *
  * Copyright (C) 2003-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -44,13 +44,34 @@ extern "C" {
 #include <dmalloc.h>
 #endif
 
-/* Can be over-ridden or undefined */
+/* Can be over-ridden or undefined in a config.h file or -Ddefine */
+#ifndef RASQAL_INLINE
 #define RASQAL_INLINE inline
+#endif
 
+#ifdef LIBRDF_DEBUG
+#define RASQAL_DEBUG 1
+#endif
+
+#if defined(RASQAL_MEMORY_SIGN)
+#define RASQAL_SIGN_KEY 0x08A59A10
+void* rasqal_sign_malloc(size_t size);
+void* rasqal_sign_calloc(size_t nmemb, size_t size);
+void* rasqal_sign_realloc(void *ptr, size_t size);
+void rasqal_sign_free(void *ptr);
+  
+#define RASQAL_MALLOC(type, size)   rasqal_sign_malloc(size)
+#define RASQAL_CALLOC(type, nmemb, size) rasqal_sign_calloc(nmemb, size)
+#define RASQAL_REALLOC(type, ptr, size) rasqal_sign_realloc(ptr, size)
+#define RASQAL_FREE(type, ptr)   rasqal_sign_free(ptr)
+
+#else
 #define RASQAL_MALLOC(type, size) malloc(size)
 #define RASQAL_CALLOC(type, size, count) calloc(size, count)
 #define RASQAL_FREE(type, ptr)   free((void*)ptr)
 
+#endif
+
 #ifdef RASQAL_DEBUG
 /* Debugging messages */
 #define RASQAL_DEBUG1(msg) do {fprintf(stderr, "%s:%d:%s: " msg, __FILE__, __LINE__, __func__); } while(0)
@@ -90,30 +111,29 @@ void rasqal_system_free(void *ptr);
 /* Fatal errors - always happen */
 #define RASQAL_FATAL1(msg) do {fprintf(stderr, "%s:%d:%s: fatal error: " msg, __FILE__, __LINE__ , __func__); abort();} while(0)
 #define RASQAL_FATAL2(msg,arg) do {fprintf(stderr, "%s:%d:%s: fatal error: " msg, __FILE__, __LINE__ , __func__, arg); abort();} while(0)
+#define RASQAL_FATAL3(msg,arg1,arg2) do {fprintf(stderr, "%s:%d:%s: fatal error: " msg, __FILE__, __LINE__ , __func__, arg1, arg2); abort();} while(0)
+
+#define RASQAL_DEPRECATED_MESSAGE(msg) do {static int warning_given=0; if(!warning_given++) fprintf(stderr, "Function %s is deprecated - " msg,  __func__); } while(0)
+#define RASQAL_DEPRECATED_WARNING(rq, msg) do {static int warning_given=0; if(!warning_given++) rasqal_query_warning(rq, msg); } while(0)
 
 
 typedef struct rasqal_query_engine_factory_s rasqal_query_engine_factory;
 
 
-/* Extra flags for patterns */
-typedef enum {
-  /* true when the pattern is optional match */
-  RASQAL_PATTERN_FLAGS_OPTIONAL=1,
-  RASQAL_PATTERN_FLAGS_LAST=RASQAL_PATTERN_FLAGS_OPTIONAL
-} rasqal_pattern_flags;
-
-
 /*
  * Pattern graph for executing
  */
 struct rasqal_graph_pattern_s {
-  rasqal_query *query;
+  rasqal_query* query;
+
+  /* operator for this graph pattern's contents */
+  rasqal_graph_pattern_operator op;
   
-  raptor_sequence *triples;          /* ... rasqal_triple*         */
-  raptor_sequence *graph_patterns;   /* ... rasqal_graph_pattern*  */
+  raptor_sequence* triples;          /* ... rasqal_triple*         */
+  raptor_sequence* graph_patterns;   /* ... rasqal_graph_pattern*  */
 
   /* An array of items, one per triple in the pattern graph */
-  rasqal_triple_meta *triple_meta;
+  rasqal_triple_meta* triple_meta;
 
   int column;
 
@@ -123,8 +143,16 @@ struct rasqal_graph_pattern_s {
   /* first graph_pattern in sequence with flags RASQAL_TRIPLE_FLAGS_OPTIONAL */
   int optional_graph_pattern;
 
-  /* enum rasqal_pattern_flags */
-  int flags;
+  /* current position in the sequence */
+  int current_graph_pattern;
+
+  /* Max optional graph pattern allowed so far to stop backtracking
+   * going over old graph patterns
+   */
+  int max_optional_graph_pattern;
+
+  /* Count of all optional matches for the current mandatory matches */
+  int optional_graph_pattern_matches_count;
 
   /* true when this graph pattern matched last time */
   int matched;
@@ -134,17 +162,26 @@ struct rasqal_graph_pattern_s {
 
   /* Number of matches returned */
   int matches_returned;
+
+  raptor_sequence *constraints; /* ... rasqal_expression*          */
+  /* the expression version of the sequence of constraints above - this is
+   * where the constraints are freed
+   */
+  rasqal_expression* constraints_expression;
+
+
 };
 
-typedef struct rasqal_graph_pattern_s rasqal_graph_pattern;
-
-
-rasqal_graph_pattern* rasqal_new_graph_pattern_from_triples(rasqal_query *query, raptor_sequence *triples, int start_column, int end_column, int flags);
-rasqal_graph_pattern* rasqal_new_graph_pattern_from_sequence(rasqal_query *query, raptor_sequence *graph_patterns, int flags);
-void rasqal_free_graph_pattern(rasqal_graph_pattern* pg);
-void rasqal_graph_pattern_init(rasqal_graph_pattern *gp);
+rasqal_graph_pattern* rasqal_new_graph_pattern(rasqal_query* query);
+rasqal_graph_pattern* rasqal_new_graph_pattern_from_triples(rasqal_query* query, raptor_sequence* triples, int start_column, int end_column, rasqal_graph_pattern_operator op);
+rasqal_graph_pattern* rasqal_new_graph_pattern_from_sequence(rasqal_query* query, raptor_sequence* graph_patterns, rasqal_graph_pattern_operator op);
+void rasqal_free_graph_pattern(rasqal_graph_pattern* gp);
+void rasqal_graph_pattern_init(rasqal_graph_pattern* gp);
 void rasqal_graph_pattern_adjust(rasqal_graph_pattern* gp, int offset);
-void rasqal_graph_pattern_print(rasqal_graph_pattern* gp, FILE* fh);
+void rasqal_graph_pattern_set_origin(rasqal_graph_pattern* graph_pattern, rasqal_literal* origin);
+void rasqal_graph_pattern_add_triples(rasqal_graph_pattern* gp, raptor_sequence* triples, int start_column, int end_column, rasqal_graph_pattern_operator op);
+int rasqal_reset_triple_meta(rasqal_triple_meta* m);
+
 
 /*
  * A query in some query language
@@ -152,59 +189,67 @@ void rasqal_graph_pattern_print(rasqal_graph_pattern* gp, FILE* fh);
 struct rasqal_query_s {
   int usage; /* reference count - 1 for itself, plus for query_results */
   
-  unsigned char *query_string;
+  unsigned char* query_string;
 
-  raptor_namespace_stack *namespaces;
+  raptor_namespace_stack* namespaces;
 
+  /* query graph pattern, containing the sequence of graph_patterns below */
+  rasqal_graph_pattern* query_graph_pattern;
+  
+  /* the query verb - in SPARQL terms: SELECT, CONSTRUCT, DESCRIBE or ASK */
+  rasqal_query_verb verb;
+  
   /* sequences of ... */
-  raptor_sequence *selects;     /* ... rasqal_variable* names only */
-  raptor_sequence *sources;     /* ... raptor_uri*                 */
-  raptor_sequence *triples;     /* ... rasqal_triple*              */
-  raptor_sequence *constraints; /* ... rasqal_expression*          */
-  raptor_sequence *prefixes;    /* ... rasqal_prefix*              */
-  raptor_sequence *constructs;  /* ... rasqal_triple*        BRQL  */
-  raptor_sequence *optional_triples; /* ... rasqal_triple*   BRQL  */
-  raptor_sequence *describes;   /* ... rasqal_literal* (var or URIs) SPARQL */
+  raptor_sequence* selects;     /* ... rasqal_variable* names only */
+  raptor_sequence* data_graphs; /* ... rasqal_data_graph*          */
+  raptor_sequence* triples;     /* ... rasqal_triple*              */
+  raptor_sequence* prefixes;    /* ... rasqal_prefix*              */
+  raptor_sequence* constructs;  /* ... rasqal_triple*       SPARQL */
+  raptor_sequence* optional_triples; /* ... rasqal_triple*  SPARQL */
+  raptor_sequence* describes;   /* ... rasqal_literal* (var or URIs) SPARQL */
 
-  /* non-0 if DISTINCT was seen in SELECT or DESCRIBE */
+  /* non-0 if DISTINCT was seen in the query (SELECT or DESCRIBE) */
   int distinct;
 
-  /* non-0 if '*' was seen in SELECT or DESCRIBE (selects will be NULL) */
-  int select_all;
+  /* result limit LIMIT (>=0) or <0 if not given */
+  int limit;
 
-  /* DESCRIBE support */
-  /* non-0 if selects array above was given by DESCRIBE */
-  int select_is_describe;
+  /* result offset OFFSET (>=0) or <0 if not given */
+  int offset;
 
-  /* CONSTRUCT support */
-  /* non-0 if 'CONSTRUCT *' was seen (constructs will be NULL) */
-  int construct_all;
-
-  /* ASK support */
-  /* non-0 if 'ASK' was seen (selects, constructs will be NULL) */
-  int ask;
+  /* non-0 if '*' was seen after a verb (the appropriate list such as selects or constructs will be NULL) */
+  int wildcard;
 
   int prepared;
   int executed;
   
   /* variable name/value table built from all distinct variables seen
-   * in selects, triples, constraints.  An array of size variables_count
+   * in selects, triples, constraints and anonymous (no name, cannot
+   * be selected or refered to).  An array of size variables_count
    *
    * The first select_variables_count of this array are from the selects
    * and are typically returned to the user.
+   *
+   * Anonymous variables appear at the end of the 'variables' array but
+   * are taken from the anon_variables_sequence.
    */
-  rasqal_variable **variables;
+  rasqal_variable** variables;
   int variables_count;
   int select_variables_count;
 
   /* array of size variables_count
    * pointing to triple column where variable[i] is declared
    */
-  int *variables_declared_in;
+  int* variables_declared_in;
 
   /* holds one copy of all the variables - this is where they are freed */
   raptor_sequence* variables_sequence;
 
+  /* holds one copy of all anonymous variables - this is where they are freed */
+  raptor_sequence* anon_variables_sequence;
+
+  int anon_variables_count;
+
   /* array of variable names to bind or NULL if no variables wanted
    * (size select_variables_count+1, last NULL)
    * indexes into the names in variables_sequence above.
@@ -217,11 +262,6 @@ struct rasqal_query_s {
    */
   rasqal_literal** binding_values;
   
-  /* the expression version of the sequence of constraints above - this is
-   * where the constraints are freed
-   */
-  rasqal_expression* constraints_expression;
-
   /* can be filled with error location information */
   raptor_locator locator;
 
@@ -232,18 +272,26 @@ struct rasqal_query_s {
   int failed;
 
   /* stuff for our user */
-  void *user_data;
+  void* user_data;
 
-  void *fatal_error_user_data;
-  void *error_user_data;
-  void *warning_user_data;
+  void* fatal_error_user_data;
+  void* error_user_data;
+  void* warning_user_data;
 
   raptor_message_handler fatal_error_handler;
   raptor_message_handler error_handler;
   raptor_message_handler warning_handler;
 
+  int default_generate_bnodeid_handler_base;
+  char *default_generate_bnodeid_handler_prefix;
+  size_t default_generate_bnodeid_handler_prefix_length;
+
+  void *generate_bnodeid_handler_user_data;
+  rasqal_generate_bnodeid_handler generate_bnodeid_handler;
+
+
   /* query engine specific stuff */
-  void *context;
+  void* context;
 
   /* stopping? */
   int abort;
@@ -261,33 +309,19 @@ struct rasqal_query_s {
   rasqal_triples_source_factory* triples_source_factory;
 
   /* (linked list of) query results made from this query */
-  rasqal_query_results *results;
+  rasqal_query_results* results;
 
   /* incrementing counter for declaring prefixes in order of appearance */
   int prefix_depth;
 
-  /* sequence of rasqal_graph_pattern* */
-  raptor_sequence *graph_patterns;
-
-  /* current position in the sequence */
-  int current_graph_pattern;
-
-  /* first graph_pattern in sequence with flags RASQAL_TRIPLE_FLAGS_OPTIONAL */
-  int optional_graph_pattern;
-
-  /* Count of all optional matches for the current mandatory matches */
-  int optional_graph_pattern_matches_count;
-
   /* New variables bound from during the current 'next result' run */
   int new_bindings_count;
 
-  /* Max optional graph pattern allowed so far to stop backtracking
-   * going over old graph patterns
-   */
-  int max_optional_graph_pattern;
-
   /* result triple - internal, not returned */
-  rasqal_triple *triple;
+  rasqal_triple* triple;
+  
+  /* sequence of constraints - internal for RDQL parsing, not returned */
+  raptor_sequence* constraints_sequence;
   
   /* result triple (SHARED) */
   raptor_statement statement;
@@ -297,6 +331,13 @@ struct rasqal_query_s {
 
   /* boolean ASK result >0 true, 0 false or -1 uninitialised */
   int ask_result;
+
+  /* sequence of order condition expressions */
+  raptor_sequence* order_conditions_sequence;
+
+  /* INTERNAL sequence of results for ordering */
+  raptor_sequence* results_sequence;
+
 };
 
 
@@ -335,7 +376,7 @@ struct rasqal_query_engine_factory_s {
   int (*prepare)(rasqal_query* rq);
   
   /* execute a query */
-  int (*execute)(rasqal_query* rq);
+  int (*execute)(rasqal_query* rq, rasqal_query_results* results);
 
   /* finish the query engine factory */
   void (*finish_factory)(rasqal_query_engine_factory* factory);
@@ -347,7 +388,7 @@ struct rasqal_query_engine_factory_s {
  */
 struct rasqal_query_results_s {
   /* query that this was executed over */
-  rasqal_query *query;
+  rasqal_query* query;
 
   /* next query result */
   rasqal_query_results *next;
@@ -355,22 +396,24 @@ struct rasqal_query_results_s {
     
 
 /* rasqal_general.c */
-char* rasqal_vsnprintf(const char *message, va_list arguments);
+char* rasqal_vsnprintf(const char* message, va_list arguments);
 
-void rasqal_query_engine_register_factory(const char *name, const char *label, const char *alias, const unsigned char *uri_string, void (*factory) (rasqal_query_engine_factory*));
-rasqal_query_engine_factory* rasqal_get_query_engine_factory (const char *name, const unsigned char *uri);
+void rasqal_query_engine_register_factory(const char* name, const char* label, const char* alias, const unsigned char* uri_string, void (*factory) (rasqal_query_engine_factory*));
+rasqal_query_engine_factory* rasqal_get_query_engine_factory (const char* name, const unsigned char* uri);
 
-void rasqal_query_fatal_error(rasqal_query* query, const char *message, ...);
-void rasqal_query_fatal_error_varargs(rasqal_query* query, const char *message, va_list arguments);
-void rasqal_query_error(rasqal_query* query, const char *message, ...);
-void rasqal_query_simple_error(void* query, const char *message, ...);
-void rasqal_query_error_varargs(rasqal_query* query, const char *message, va_list arguments);
-void rasqal_query_warning(rasqal_query* query, const char *message, ...);
-void rasqal_query_warning_varargs(rasqal_query* query, const char *message, va_list arguments);
+void rasqal_query_fatal_error(rasqal_query* query, const char* message, ...);
+void rasqal_query_fatal_error_varargs(rasqal_query* query, const char* message, va_list arguments);
+void rasqal_query_error(rasqal_query* query, const char* message, ...);
+void rasqal_query_simple_error(void* query, const char* message, ...);
+void rasqal_query_error_varargs(rasqal_query* query, const char* message, va_list arguments);
+void rasqal_query_warning(rasqal_query* query, const char* message, ...);
+void rasqal_query_warning_varargs(rasqal_query* query, const char* message, va_list arguments);
 
-const char* rasqal_basename(const char *name);
+const char* rasqal_basename(const char* name);
 
-unsigned char* rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len, size_t *dest_lenp, raptor_simple_message_handler error_handler, void *error_data);
+unsigned char* rasqal_escaped_name_to_utf8_string(const unsigned char* src, size_t len, size_t* dest_lenp, raptor_simple_message_handler error_handler, void* error_data);
+
+unsigned char* rasqal_query_generate_bnodeid(rasqal_query* rdf_query, unsigned char *user_bnodeid);
 
 /* rdql_parser.y */
 void rasqal_init_query_engine_rdql (void);
@@ -380,38 +423,41 @@ void rasqal_init_query_engine_sparql (void);
 
 /* rasqal_engine.c */
 int rasqal_query_order_triples(rasqal_query* query);
-int rasqal_engine_declare_prefix(rasqal_query *rq, rasqal_prefix *prefix);
-int rasqal_engine_undeclare_prefix(rasqal_query *rq, rasqal_prefix *prefix);
-int rasqal_engine_declare_prefixes(rasqal_query *rq);
-int rasqal_engine_sequence_has_qname(raptor_sequence *seq);
+int rasqal_engine_declare_prefix(rasqal_query* rq, rasqal_prefix* prefix);
+int rasqal_engine_undeclare_prefix(rasqal_query* rq, rasqal_prefix* prefix);
+int rasqal_engine_declare_prefixes(rasqal_query* rq);
+int rasqal_engine_sequence_has_qname(raptor_sequence* seq);
 int rasqal_engine_expand_triple_qnames(rasqal_query* rq);
-int rasqal_engine_constraints_has_qname(rasqal_query* rq);
-int rasqal_engine_expand_constraints_qnames(rasqal_query* rq);
-int rasqal_engine_build_constraints_expression(rasqal_query* rq);
+int rasqal_engine_query_constraints_has_qname(rasqal_query* gp);
+int rasqal_engine_graph_pattern_constraints_has_qname(rasqal_graph_pattern* gp);
+int rasqal_engine_expand_query_constraints_qnames(rasqal_query* rq);
+int rasqal_engine_expand_graph_pattern_constraints_qnames(rasqal_query* rq, rasqal_graph_pattern* gp);
+int rasqal_engine_build_constraints_expression(rasqal_graph_pattern* gp);
 int rasqal_engine_assign_variables(rasqal_query* rq);
 
-int rasqal_engine_prepare(rasqal_query *query);
-int rasqal_engine_execute_init(rasqal_query *query);
-int rasqal_engine_execute_finish(rasqal_query *query);
-int rasqal_engine_run(rasqal_query *q);
+int rasqal_engine_prepare(rasqal_query* query);
+int rasqal_engine_execute_init(rasqal_query* query, rasqal_query_results* query_results);
+int rasqal_engine_execute_finish(rasqal_query* query);
+int rasqal_engine_run(rasqal_query* q);
+void rasqal_engine_make_basic_graph_pattern(rasqal_graph_pattern *gp);
 
-rasqal_triples_source* rasqal_new_triples_source(rasqal_query *query);
-void rasqal_free_triples_source(rasqal_triples_source *rts);
-int rasqal_triples_source_next_source(rasqal_triples_source *rts);
+rasqal_triples_source* rasqal_new_triples_source(rasqal_query* query);
+void rasqal_free_triples_source(rasqal_triples_source* rts);
+int rasqal_triples_source_next_source(rasqal_triples_source* rts);
 
-int rasqal_engine_get_next_result(rasqal_query *query);
-void rasqal_engine_assign_binding_values(rasqal_query *query);
+int rasqal_engine_get_next_result(rasqal_query* query);
+void rasqal_engine_assign_binding_values(rasqal_query* query);
 
 /* rasqal_expr.c */
-int rasqal_literal_as_boolean(rasqal_literal* literal, int *error);
-int rasqal_literal_as_integer(rasqal_literal* l, int *error);
-double rasqal_literal_as_floating(rasqal_literal* l, int *error);
+int rasqal_literal_as_boolean(rasqal_literal* literal, int* error);
+int rasqal_literal_as_integer(rasqal_literal* l, int* error);
+double rasqal_literal_as_floating(rasqal_literal* l, int* error);
 raptor_uri* rasqal_literal_as_uri(rasqal_literal* l);
-void rasqal_literal_string_to_native(rasqal_literal *l);
-int rasqal_literal_has_qname(rasqal_literal *l);
-int rasqal_literal_expand_qname(void *user_data, rasqal_literal *l);
-int rasqal_expression_has_qname(void *user_data, rasqal_expression *e);
-int rasqal_expression_expand_qname(void *user_data, rasqal_expression *e);
+void rasqal_literal_string_to_native(rasqal_literal* l);
+int rasqal_literal_has_qname(rasqal_literal* l);
+int rasqal_literal_expand_qname(void* user_data, rasqal_literal* l);
+int rasqal_expression_has_qname(void* user_data, rasqal_expression* e);
+int rasqal_expression_expand_qname(void* user_data, rasqal_expression* e);
 
 /* strcasecmp.c */
 #ifdef HAVE_STRCASECMP
@@ -433,6 +479,30 @@ void rasqal_redland_init(void);
 void rasqal_redland_finish(void);
 #endif  
 
+
+/* raptor_general.c */
+extern raptor_uri* rasqal_xsd_namespace_uri;
+extern raptor_uri* rasqal_xsd_integer_uri;
+extern raptor_uri* rasqal_xsd_double_uri;
+extern raptor_uri* rasqal_xsd_boolean_uri;
+extern raptor_uri* rasqal_rdf_namespace_uri;
+extern raptor_uri* rasqal_rdf_first_uri;
+extern raptor_uri* rasqal_rdf_rest_uri;
+extern raptor_uri* rasqal_rdf_nil_uri;
+void rasqal_uri_init(void);
+void rasqal_uri_finish(void);
+
+/* rasqal_literal.c */
+typedef struct {
+  raptor_sequence *triples;
+  rasqal_literal *value;
+} rasqal_formula;
+
+rasqal_formula* rasqal_new_formula(void);
+void rasqal_free_formula(rasqal_formula* formula);
+void rasqal_formula_print(rasqal_formula* formula, FILE *stream);
+
+
 /* end of RASQAL_INTERNAL */
 #endif
 
