@@ -2,7 +2,7 @@
  *
  * rasqal_literal.c - Rasqal literals
  *
- * $Id: rasqal_literal.c,v 1.23 2005/02/03 14:07:43 cmdjb Exp $
+ * $Id: rasqal_literal.c,v 1.32 2005/06/09 14:15:25 cmdjb Exp $
  *
  * Copyright (C) 2003-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -73,7 +73,7 @@ rasqal_new_integer_literal(rasqal_literal_type type, int integer)
   l->value.integer=integer;
   l->string=(unsigned char*)RASQAL_MALLOC(cstring, 30); /* FIXME */
   sprintf((char*)l->string, "%d", integer);
-  l->datatype=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/XMLSchema#integer");
+  l->datatype=raptor_uri_copy(rasqal_xsd_integer_uri);
   l->usage=1;
   return l;
 }
@@ -94,7 +94,7 @@ rasqal_new_floating_literal(double f)
   l->value.floating=f;
   l->string=(unsigned char*)RASQAL_MALLOC(cstring, 30); /* FIXME */
   sprintf((char*)l->string, "%1g", f);
-  l->datatype=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/XMLSchema#double");
+  l->datatype=raptor_uri_copy(rasqal_xsd_double_uri);
   l->usage=1;
   return l;
 }
@@ -146,11 +146,13 @@ rasqal_new_pattern_literal(const unsigned char *pattern,
 
 
 /*
- * rasqal_literal_string_to_native - INTERNAL Upgrade a literal string to a datatyped literal
+ * rasqal_literal_string_to_native - INTERNAL Upgrade a datatyped literal string to an internal typed literal
  * @l: &rasqal_literal to operate on inline
  *
- * At present this promotes xsd:decimal to RASQAL_LITERAL_INTEGER and
- * xsd:double to RASQAL_INTEGER_FLOATING.
+ * At present this promotes datatyped literals
+ * xsd:integer to RASQAL_LITERAL_INTEGER
+ * xsd:double to RASQAL_LITERAL_FLOATING
+ * xsd:boolean to RASQAL_LITERAL_BOOLEAN
  **/
 void
 rasqal_literal_string_to_native(rasqal_literal *l)
@@ -158,11 +160,11 @@ rasqal_literal_string_to_native(rasqal_literal *l)
   if(!l->datatype)
     return;
 
-  if(!strcmp((const char*)raptor_uri_as_string(l->datatype), "http://www.w3.org/2001/XMLSchema#integer")) {
+  if(raptor_uri_equals(l->datatype, rasqal_xsd_integer_uri)) {
     int i=atoi((const char*)l->string);
 
     if(l->language) {
-      RASQAL_FREE(cstring, l->language);
+      RASQAL_FREE(cstring, (void*)l->language);
       l->language=NULL;
     }
 
@@ -171,12 +173,12 @@ rasqal_literal_string_to_native(rasqal_literal *l)
     return;
   }
   
-  if(!strcmp((const char*)raptor_uri_as_string(l->datatype), "http://www.w3.org/2001/XMLSchema#double")) {
+  if(raptor_uri_equals(l->datatype, rasqal_xsd_double_uri)) {
     double d=0.0;
     sscanf((char*)l->string, "%lf", &d);
 
     if(l->language) {
-      RASQAL_FREE(cstring, l->language);
+      RASQAL_FREE(cstring, (void*)l->language);
       l->language=NULL;
     }
 
@@ -184,6 +186,26 @@ rasqal_literal_string_to_native(rasqal_literal *l)
     l->value.floating=d;
     return;
   }
+
+  if(raptor_uri_equals(l->datatype, rasqal_xsd_boolean_uri)) {
+    int b=0;
+    if(!strcmp((const char*)l->string, "true") || 
+       !strcmp((const char*)l->string, "TRUE"))
+       b=1;
+    
+    if(l->language) {
+      RASQAL_FREE(cstring, (void*)l->language);
+      l->language=NULL;
+    }
+
+    /* static string for boolean */
+    l->string=(const unsigned char*)(b ? "true":"false");
+
+    l->type=RASQAL_LITERAL_BOOLEAN;
+    l->value.integer=b;
+    return;
+  }
+
 }
 
 
@@ -216,7 +238,7 @@ rasqal_new_string_literal(const unsigned char *string,
   rasqal_literal* l=(rasqal_literal*)RASQAL_CALLOC(rasqal_literal, sizeof(rasqal_literal), 1);
 
   if(datatype && language) {
-    RASQAL_FREE(cstring, language);
+    RASQAL_FREE(cstring, (void*)language);
     language=NULL;
   }
 
@@ -268,6 +290,7 @@ rasqal_new_boolean_literal(int value)
 
   l->type=RASQAL_LITERAL_BOOLEAN;
   l->value.integer=value;
+  /* static string for boolean */
   l->string=(const unsigned char*)(value ? "true":"false");
   l->usage=1;
   return l;
@@ -330,19 +353,22 @@ rasqal_free_literal(rasqal_literal* l)
     case RASQAL_LITERAL_FLOATING:
     case RASQAL_LITERAL_INTEGER: 
      if(l->string)
-        RASQAL_FREE(cstring,l->string);
+        RASQAL_FREE(cstring, (void*)l->string);
       if(l->language)
-        RASQAL_FREE(cstring,l->language);
+        RASQAL_FREE(cstring, (void*)l->language);
       if(l->datatype)
         raptor_free_uri(l->datatype);
       if(l->type == RASQAL_LITERAL_STRING ||
          l->type == RASQAL_LITERAL_PATTERN) {
         if(l->flags)
-          RASQAL_FREE(cstring, l->flags);
+          RASQAL_FREE(cstring, (void*)l->flags);
       }
       break;
+
     case RASQAL_LITERAL_BOOLEAN:
+      /* static l->string for boolean, does not need freeing */
       break;
+
     case RASQAL_LITERAL_VARIABLE:
       /* It is correct that this is not called here
        * since all variables are shared and owned by
@@ -351,6 +377,7 @@ rasqal_free_literal(rasqal_literal* l)
       /* rasqal_free_variable(l->value.variable); */
       break;
 
+    case RASQAL_LITERAL_UNKNOWN:
     default:
       abort();
   }
@@ -442,6 +469,8 @@ rasqal_literal_print(rasqal_literal* l, FILE* fh)
     case RASQAL_LITERAL_VARIABLE:
       rasqal_variable_print(l->value.variable, fh);
       break;
+
+    case RASQAL_LITERAL_UNKNOWN:
     default:
       abort();
   }
@@ -490,6 +519,7 @@ rasqal_literal_as_boolean(rasqal_literal* l, int *error)
       return rasqal_literal_as_boolean(l->value.variable->value, error);
       break;
 
+    case RASQAL_LITERAL_UNKNOWN:
     default:
       abort();
   }
@@ -547,6 +577,14 @@ rasqal_literal_as_integer(rasqal_literal* l, int *error)
       return rasqal_literal_as_integer(l->value.variable->value, error);
       break;
 
+    case RASQAL_LITERAL_BLANK:
+    case RASQAL_LITERAL_URI:
+    case RASQAL_LITERAL_QNAME:
+    case RASQAL_LITERAL_PATTERN:
+      *error=1;
+      return 0;
+      
+    case RASQAL_LITERAL_UNKNOWN:
     default:
       abort();
   }
@@ -588,13 +626,21 @@ rasqal_literal_as_floating(rasqal_literal* l, int *error)
           return d;
       }
       *error=1;
-      return 0;
+      return 0.0;
       break;
 
     case RASQAL_LITERAL_VARIABLE:
       return rasqal_literal_as_integer(l->value.variable->value, error);
       break;
 
+    case RASQAL_LITERAL_BLANK:
+    case RASQAL_LITERAL_URI:
+    case RASQAL_LITERAL_QNAME:
+    case RASQAL_LITERAL_PATTERN:
+      *error=1;
+      return 0.0;
+      
+    case RASQAL_LITERAL_UNKNOWN:
     default:
       abort();
   }
@@ -643,6 +689,8 @@ rasqal_literal_as_string(rasqal_literal* l)
     case RASQAL_LITERAL_FLOATING:
     case RASQAL_LITERAL_STRING:
     case RASQAL_LITERAL_BLANK:
+    case RASQAL_LITERAL_PATTERN:
+    case RASQAL_LITERAL_QNAME:
       return l->string;
 
     case RASQAL_LITERAL_URI:
@@ -651,6 +699,7 @@ rasqal_literal_as_string(rasqal_literal* l)
     case RASQAL_LITERAL_VARIABLE:
       return rasqal_literal_as_string(l->value.variable->value);
 
+    case RASQAL_LITERAL_UNKNOWN:
     default:
       abort();
   }
@@ -694,8 +743,7 @@ double_to_int(double d)
  *
  * The comparison returned is as for strcmp, first before second
  * returns <0.  equal returns 0, and first after second returns >0.
- * If there is no ordering, such as for URIs, the return value
- * is 0 for equal, non-0 for different (using raptor_uri_equals).
+ * For URIs, the string value is used for the comparsion.
  *
  * flags affects string comparisons and if the
  * RASQAL_COMPARE_NOCASE bit is set, a case independent
@@ -733,14 +781,26 @@ rasqal_literal_compare(rasqal_literal* l1, rasqal_literal* l2, int flags,
   for(i=0; i<2; i++) {
     if(lits[i]->type == RASQAL_LITERAL_VARIABLE) {
       lits[i]=lits[i]->value.variable->value;
+
+      /* Need to re-check for NULL values */
+      if(!lits[i]) {
+        /* A null value, so the comparison fails */
+        RASQAL_DEBUG2("literal %d is a variable with no value\n", i);
+        if(lits[1-i])
+          *error=1;
+        return 0;
+      }
+
       RASQAL_DEBUG3("literal %d is a variable, value is a %s\n", i,
                     rasqal_literal_type_labels[lits[i]->type]);
+
     }
     
 
     switch(lits[i]->type) {
       case RASQAL_LITERAL_URI:
         break;
+
       case RASQAL_LITERAL_STRING:
       case RASQAL_LITERAL_BLANK:
       case RASQAL_LITERAL_PATTERN:
@@ -762,6 +822,10 @@ rasqal_literal_compare(rasqal_literal* l1, rasqal_literal* l2, int flags,
         seen_double=1;
         break;
 
+    case RASQAL_LITERAL_VARIABLE:
+      /* this case was dealt with above, retrieving the value */
+
+    case RASQAL_LITERAL_UNKNOWN:
       default:
         abort();
     }
@@ -832,7 +896,8 @@ rasqal_literal_compare(rasqal_literal* l1, rasqal_literal* l2, int flags,
 
   switch(type) {
     case RASQAL_LITERAL_URI:
-      return !raptor_uri_equals(lits[0]->value.uri,lits[1]->value.uri);
+      return strcmp((const char*)raptor_uri_as_string(lits[0]->value.uri),
+                    (const char*)raptor_uri_as_string(lits[1]->value.uri));
 
     case RASQAL_LITERAL_STRING:
       if(lits[0]->language || lits[1]->language) {
@@ -844,11 +909,17 @@ rasqal_literal_compare(rasqal_literal* l1, rasqal_literal* l2, int flags,
       }
 
       if(lits[0]->datatype || lits[1]->datatype) {
-        /* if either is null, the comparison fails */
+        int result;
+        
+        /* if either is NULL, do not compare */
         if(!lits[0]->datatype || !lits[1]->datatype)
-          return 1;
-        if(!raptor_uri_equals(lits[0]->datatype,lits[1]->datatype))
-          return 1;
+          return lits[0]->datatype ? 1 : -1;
+
+        result=strcmp((const char*)raptor_uri_as_string(lits[0]->datatype),
+                      (const char*)raptor_uri_as_string(lits[1]->datatype));
+
+        if(result)
+          return result;
       }
       
       /* FALLTHROUGH */
@@ -938,6 +1009,12 @@ rasqal_literal_equals(rasqal_literal* l1, rasqal_literal* l2)
       return l1->value.floating == l2->value.floating;
       break;
 
+    case RASQAL_LITERAL_VARIABLE:
+      /* both are variables */
+      return rasqal_literal_equals(l1->value.variable->value,
+                                   l2->value.variable->value);
+      
+    case RASQAL_LITERAL_UNKNOWN:
     default:
       abort();
   }
@@ -970,7 +1047,7 @@ rasqal_literal_expand_qname(void *user_data, rasqal_literal *l)
                                                rasqal_query_simple_error, rq);
     if(!uri)
       return 1;
-    RASQAL_FREE(cstring, l->string);
+    RASQAL_FREE(cstring, (void*)l->string);
     l->string=NULL;
     l->type=RASQAL_LITERAL_URI;
     l->value.uri=uri;
@@ -986,11 +1063,11 @@ rasqal_literal_expand_qname(void *user_data, rasqal_literal *l)
       if(!uri)
         return 1;
       l->datatype=uri;
-      RASQAL_FREE(cstring, l->flags);
+      RASQAL_FREE(cstring, (void*)l->flags);
       l->flags=NULL;
 
       if(l->language && uri) {
-        RASQAL_FREE(cstring, l->language);
+        RASQAL_FREE(cstring, (void*)l->language);
         l->language=NULL;
       }
 
@@ -1036,14 +1113,21 @@ rasqal_literal_as_node(rasqal_literal* l)
       break;
       
     case RASQAL_LITERAL_VARIABLE:
-      return rasqal_new_literal_from_literal(l->value.variable->value);
+      new_l=l->value.variable->value;
+      if(new_l)
+        return rasqal_new_literal_from_literal(new_l);
+      else
+        return NULL;
+      break;
 
     case RASQAL_LITERAL_FLOATING:
     case RASQAL_LITERAL_INTEGER:
-      dt_uri=raptor_uri_copy(l->datatype);
-      /* FALLTHROUGH */
+    case RASQAL_LITERAL_BOOLEAN:
+      if(l->type == RASQAL_LITERAL_BOOLEAN)
+        dt_uri=raptor_uri_copy(rasqal_xsd_boolean_uri);
+      else
+        dt_uri=raptor_uri_copy(l->datatype);
 
-    default:
       new_l=(rasqal_literal*)RASQAL_CALLOC(rasqal_literal, sizeof(rasqal_literal), 1);
 
       new_l->type=RASQAL_LITERAL_STRING;
@@ -1052,7 +1136,54 @@ rasqal_literal_as_node(rasqal_literal* l)
       new_l->datatype=dt_uri;
       new_l->flags=NULL;
       new_l->usage=1;
+      break;
+      
+    case RASQAL_LITERAL_QNAME:
+      /* QNames should be gone by the time expression eval happens */
+
+    case RASQAL_LITERAL_PATTERN:
+      /* FALLTHROUGH */
+
+    case RASQAL_LITERAL_UNKNOWN:
+    default:
+      RASQAL_FATAL2("Cannot turn literal type %d into a node", l->type);
+      abort();
   }
   
   return new_l;
 }
+
+
+rasqal_formula*
+rasqal_new_formula(void) 
+{
+  return (rasqal_formula*)RASQAL_CALLOC(rasqal_formula, sizeof(rasqal_formula), 1);
+}
+
+void
+rasqal_free_formula(rasqal_formula* formula)
+{
+  if(formula->triples)
+    raptor_free_sequence(formula->triples);
+  if(formula->value)
+    rasqal_free_literal(formula->value);
+  RASQAL_FREE(rasqal_formula, formula);
+}
+  
+
+void
+rasqal_formula_print(rasqal_formula* formula, FILE *stream)
+{
+  fputs("formula(triples=", stream);
+  if(formula->triples)
+    raptor_sequence_print(formula->triples, stream);
+  else
+    fputs("[]", stream);
+  fputs(", value=", stream);
+  if(formula->value)
+    rasqal_literal_print(formula->value, stream);
+  else
+    fputs("NULL", stream);
+  fputc(')', stream);
+}
+
