@@ -2,7 +2,7 @@
  *
  * rasqal_query_test.c - Rasqal RDF Query Tests
  *
- * $Id: rasqal_query_test.c,v 1.10 2005/01/16 09:49:51 cmdjb Exp $
+ * $Id: rasqal_query_test.c,v 1.12 2005/05/17 06:40:44 cmdjb Exp $
  *
  * Copyright (C) 2004-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -98,7 +98,7 @@ main(int argc, char **argv) {
   }
     
   data_string=raptor_uri_filename_to_uri_string(argv[1]);
-  query_string=(unsigned char*)malloc(strlen((const char*)data_string)+strlen(query_format)+1);
+  query_string=(unsigned char*)RASQAL_MALLOC(cstring, strlen((const char*)data_string)+strlen(query_format)+1);
   sprintf((char*)query_string, query_format, data_string);
   raptor_free_memory(data_string);
   
@@ -120,7 +120,7 @@ main(int argc, char **argv) {
     return(1);
   }
 
-  raptor_free_memory(query_string);
+  RASQAL_FREE(cstring, query_string);
 
   printf("%s: executing query #1\n", program);
   results=rasqal_query_execute(query);
@@ -180,6 +180,24 @@ main(int argc, char **argv) {
     return(1);
   }
 
+  printf("%s: executing query #3\n", program);
+  results = rasqal_query_execute(query);
+  if(!results) {
+    fprintf(stderr, "%s: query execution 3 FAILED\n", program);
+    return(1);
+  }
+
+  rasqal_free_query_results(results);
+
+  printf("%s: executing query #4\n", program);
+  results = rasqal_query_execute(query);
+  if(!results) {
+    fprintf(stderr, "%s: query execution 4 FAILED\n", program);
+    return(1);
+  }
+
+  rasqal_free_query_results(results);
+
   rasqal_free_query(query);
 
   raptor_free_uri(base_uri);
