@@ -2,7 +2,7 @@
  *
  * rasqal_raptor.c - Rasqal triple store implementation with raptor
  *
- * $Id: rasqal_raptor.c,v 1.36 2005/01/15 01:09:31 cmdjb Exp $
+ * $Id: rasqal_raptor.c,v 1.40 2005/05/17 22:22:14 cmdjb Exp $
  * 
  * Copyright (C) 2004-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -61,9 +61,9 @@ typedef struct {
   int source_index;
 
   /* size of the two arrays below */
-  int source_uris_count;
+  int sources_count;
   
-  /* array of shared pointers into query->source uris */
+  /* array of shared pointers into query->data_graph uris */
   raptor_uri **source_uris;
 
   /* array of URI literals (allocated here) */
@@ -72,7 +72,7 @@ typedef struct {
 
 
 /* prototypes */
-static rasqal_triples_match* rasqal_raptor_new_triples_match(rasqal_triples_source *rts, void *user_data, rasqal_triple_meta *m, rasqal_triple *t);
+static int rasqal_raptor_init_triples_match(rasqal_triples_match* rtm, rasqal_triples_source *rts, void *user_data, rasqal_triple_meta *m, rasqal_triple *t);
 static int rasqal_raptor_triple_present(rasqal_triples_source *rts, void *user_data, rasqal_triple *t);
 static void rasqal_raptor_free_triples_source(void *user_data);
 
@@ -220,27 +220,27 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
   const char *parser_name;
   int i;
 
-  if(!rdf_query->sources)
-    return 1;
+  if(!rdf_query->data_graphs)
+    return -1;  /* no data */
 
-  rts->new_triples_match=rasqal_raptor_new_triples_match;
+  rts->init_triples_match=rasqal_raptor_init_triples_match;
   rts->triple_present=rasqal_raptor_triple_present;
   rts->free_triples_source=rasqal_raptor_free_triples_source;
 
-  rtsc->source_uris_count=raptor_sequence_size(rdf_query->sources);
+  rtsc->sources_count=raptor_sequence_size(rdf_query->data_graphs);
   /* no default triple source possible */
-  if(!rtsc->source_uris_count)
-    return 1;
+  if(!rtsc->sources_count)
+    return -1;  /* no data */
 
-  rtsc->source_uris=(raptor_uri**)RASQAL_CALLOC(raptor_uri_ptr, rtsc->source_uris_count, sizeof(raptor_uri*));
-  rtsc->source_literals=(rasqal_literal**)RASQAL_CALLOC(rasqal_literal_ptr, rtsc->source_uris_count, sizeof(rasqal_literal*));
+  rtsc->source_uris=(raptor_uri**)RASQAL_CALLOC(raptor_uri_ptr, rtsc->sources_count, sizeof(raptor_uri*));
+  rtsc->source_literals=(rasqal_literal**)RASQAL_CALLOC(rasqal_literal_ptr, rtsc->sources_count, sizeof(rasqal_literal*));
 
-  for(i=0; i< rtsc->source_uris_count; i++) {
-    raptor_uri* uri=(raptor_uri*)raptor_sequence_get_at(rdf_query->sources, i);
+  for(i=0; i< rtsc->sources_count; i++) {
+    rasqal_data_graph *dg=(rasqal_data_graph*)raptor_sequence_get_at(rdf_query->data_graphs, i);
+    raptor_uri* uri=dg->uri;
 
     rtsc->source_index=i;
-    /* not allocated here; these are shared pointers into query->sources */
-    rtsc->source_uris[i]=uri;
+    rtsc->source_uris[i]=raptor_uri_copy(uri);
     rtsc->source_literals[i]=rasqal_new_uri_literal(raptor_uri_copy(uri));
 
     parser_name=raptor_guess_parser_name(NULL, NULL, NULL, 0, 
@@ -248,7 +248,7 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
     parser=raptor_new_parser(parser_name);
     raptor_set_statement_handler(parser, rtsc, rasqal_raptor_statement_handler);
     raptor_set_error_handler(parser, rdf_query, rasqal_raptor_error_handler);
-    raptor_parse_uri(parser, uri, NULL);
+    raptor_parse_uri(parser, uri, dg->name_uri);
     raptor_free_parser(parser);
     if(rdf_query->failed) {
       rasqal_raptor_free_triples_source(user_data);
@@ -330,9 +330,8 @@ rasqal_raptor_free_triples_source(void *user_data) {
     cur=next;
   }
 
-  for(i=0; i< rtsc->source_uris_count; i++) {
-    /* not freed here; these are shared pointers into query->sources */
-    /* raptor_free_uri(rtsc->source_uris[i]); */
+  for(i=0; i< rtsc->sources_count; i++) {
+    raptor_free_uri(rtsc->source_uris[i]);
     rasqal_free_literal(rtsc->source_literals[i]);
   }
   RASQAL_FREE(raptor_uri_ptr, rtsc->source_uris);
@@ -490,15 +489,14 @@ rasqal_raptor_finish_triples_match(struct rasqal_triples_match_s* rtm,
 }
 
 
-static rasqal_triples_match*
-rasqal_raptor_new_triples_match(rasqal_triples_source *rts, void *user_data,
-                                rasqal_triple_meta *m, rasqal_triple *t) {
+static int
+rasqal_raptor_init_triples_match(rasqal_triples_match* rtm,
+                                 rasqal_triples_source *rts, void *user_data,
+                                 rasqal_triple_meta *m, rasqal_triple *t) {
   rasqal_raptor_triples_source_user_data* rtsc=(rasqal_raptor_triples_source_user_data*)user_data;
-  rasqal_triples_match *rtm;
   rasqal_raptor_triples_match_context* rtmc;
   rasqal_variable* var;
 
-  rtm=(rasqal_triples_match *)RASQAL_CALLOC(rasqal_triples_match, sizeof(rasqal_triples_match), 1);
   rtm->bind_match=rasqal_raptor_bind_match;
   rtm->next_match=rasqal_raptor_next_match;
   rtm->is_end=rasqal_raptor_is_end;
@@ -559,9 +557,9 @@ rasqal_raptor_new_triples_match(rasqal_triples_source *rts, void *user_data,
   }
   
     
-  RASQAL_DEBUG1("rasqal_new_triples_match done\n");
+  RASQAL_DEBUG1("rasqal_init_triples_match done\n");
 
-  return rtm;
+  return 0;
 }
 
 
