@@ -144,6 +144,7 @@
 #include <rdql_parser.h>
 
 #define YY_DECL int rdql_lexer_lex (YYSTYPE *rdql_parser_lval, yyscan_t yyscanner)
+#define YY_NO_UNISTD_H 1
 #include <rdql_lexer.h>
 
 #include <rdql_common.h>
@@ -206,7 +207,7 @@ static int rdql_query_error(rasqal_query* rq, const char *message);
 #endif
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 102 "./rdql_parser.y"
+#line 103 "./rdql_parser.y"
 typedef union YYSTYPE {
   raptor_sequence *seq;
   rasqal_variable *variable;
@@ -218,7 +219,7 @@ typedef union YYSTYPE {
   unsigned char *name;
 } YYSTYPE;
 /* Line 191 of yacc.c.  */
-#line 222 "rdql_parser.c"
+#line 223 "rdql_parser.c"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -230,7 +231,7 @@ typedef union YYSTYPE {
 
 
 /* Line 214 of yacc.c.  */
-#line 234 "rdql_parser.c"
+#line 235 "rdql_parser.c"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
@@ -436,14 +437,14 @@ static const yysigned_char yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned short yyrline[] =
 {
-       0,   173,   173,   177,   194,   199,   204,   213,   217,   224,
-     228,   233,   242,   247,   252,   265,   269,   273,   277,   288,
-     293,   298,   303,   308,   317,   322,   327,   332,   337,   345,
-     349,   355,   360,   366,   370,   374,   378,   382,   388,   392,
-     396,   402,   406,   410,   414,   418,   424,   431,   435,   439,
-     445,   449,   453,   457,   463,   467,   471,   477,   481,   485,
-     490,   494,   500,   504,   508,   514,   518,   524,   530,   536,
-     540,   544,   548,   552,   556,   559,   566,   571
+       0,   174,   174,   178,   201,   206,   211,   220,   224,   231,
+     235,   240,   249,   254,   259,   272,   276,   280,   284,   295,
+     300,   305,   310,   315,   324,   329,   334,   339,   344,   352,
+     356,   362,   367,   373,   377,   381,   385,   389,   395,   399,
+     403,   409,   413,   417,   421,   425,   431,   438,   442,   446,
+     452,   456,   460,   464,   470,   474,   478,   484,   488,   492,
+     497,   501,   507,   511,   515,   521,   525,   531,   537,   543,
+     547,   551,   555,   559,   563,   566,   573,   578
 };
 #endif
 
@@ -1234,12 +1235,18 @@ yyreduce:
   switch (yyn)
     {
         case 3:
-#line 178 "./rdql_parser.y"
+#line 179 "./rdql_parser.y"
     {
   ((rasqal_query*)rq)->selects=yyvsp[-5].seq;
+  ((rasqal_query*)rq)->verb=RASQAL_QUERY_VERB_SELECT;
 
   if(yyvsp[-4].seq) {
-    raptor_sequence_join(((rasqal_query*)rq)->sources, yyvsp[-4].seq);
+    int i;
+    
+    for(i=0; i < raptor_sequence_size(yyvsp[-4].seq); i++) {
+      raptor_uri* uri=(raptor_uri*)raptor_sequence_get_at(yyvsp[-4].seq, i);
+      rasqal_query_add_data_graph((rasqal_query*)rq, uri, NULL, RASQAL_DATA_GRAPH_BACKGROUND);
+    }
     raptor_free_sequence(yyvsp[-4].seq);
   }
 
@@ -1252,7 +1259,7 @@ yyreduce:
     break;
 
   case 4:
-#line 195 "./rdql_parser.y"
+#line 202 "./rdql_parser.y"
     {
   yyval.seq=yyvsp[-2].seq;
   raptor_sequence_push(yyval.seq, yyvsp[0].variable);
@@ -1260,7 +1267,7 @@ yyreduce:
     break;
 
   case 5:
-#line 200 "./rdql_parser.y"
+#line 207 "./rdql_parser.y"
     {
   yyval.seq=yyvsp[-1].seq;
   raptor_sequence_push(yyval.seq, yyvsp[0].variable);
@@ -1268,7 +1275,7 @@ yyreduce:
     break;
 
   case 6:
-#line 205 "./rdql_parser.y"
+#line 212 "./rdql_parser.y"
     {
   /* The variables are freed from the rasqal_query field variables */
   yyval.seq=raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
@@ -1277,43 +1284,43 @@ yyreduce:
     break;
 
   case 7:
-#line 214 "./rdql_parser.y"
+#line 221 "./rdql_parser.y"
     {
   yyval.seq=yyvsp[0].seq;
 }
     break;
 
   case 8:
-#line 218 "./rdql_parser.y"
+#line 225 "./rdql_parser.y"
     {
   yyval.seq=NULL;
-  ((rasqal_query*)rq)->select_all=1;
+  ((rasqal_query*)rq)->wildcard=1;
 }
     break;
 
   case 9:
-#line 225 "./rdql_parser.y"
+#line 232 "./rdql_parser.y"
     {
   yyval.seq=yyvsp[0].seq;
 }
     break;
 
   case 10:
-#line 229 "./rdql_parser.y"
+#line 236 "./rdql_parser.y"
     {
   yyval.seq=yyvsp[0].seq;
 }
     break;
 
   case 11:
-#line 233 "./rdql_parser.y"
+#line 240 "./rdql_parser.y"
     {
   yyval.seq=NULL;
 }
     break;
 
   case 12:
-#line 243 "./rdql_parser.y"
+#line 250 "./rdql_parser.y"
     {
   yyval.seq=yyvsp[-2].seq;
   raptor_sequence_push(yyval.seq, yyvsp[0].triple);
@@ -1321,7 +1328,7 @@ yyreduce:
     break;
 
   case 13:
-#line 248 "./rdql_parser.y"
+#line 255 "./rdql_parser.y"
     {
   yyval.seq=yyvsp[-1].seq;
   raptor_sequence_push(yyval.seq, yyvsp[0].triple);
@@ -1329,7 +1336,7 @@ yyreduce:
     break;
 
   case 14:
-#line 253 "./rdql_parser.y"
+#line 260 "./rdql_parser.y"
     {
   yyval.seq=((rasqal_query*)rq)->triples;
   raptor_sequence_push(yyval.seq, yyvsp[0].triple);
@@ -1337,87 +1344,87 @@ yyreduce:
     break;
 
   case 15:
-#line 266 "./rdql_parser.y"
+#line 273 "./rdql_parser.y"
     {
   yyval.triple=rasqal_new_triple(yyvsp[-5].literal, yyvsp[-3].literal, yyvsp[-1].literal);
 }
     break;
 
   case 16:
-#line 270 "./rdql_parser.y"
+#line 277 "./rdql_parser.y"
     {
   yyval.triple=rasqal_new_triple(yyvsp[-4].literal, yyvsp[-3].literal, yyvsp[-1].literal);
 }
     break;
 
   case 17:
-#line 274 "./rdql_parser.y"
+#line 281 "./rdql_parser.y"
     {
   yyval.triple=rasqal_new_triple(yyvsp[-4].literal, yyvsp[-2].literal, yyvsp[-1].literal);
 }
     break;
 
   case 18:
-#line 278 "./rdql_parser.y"
+#line 285 "./rdql_parser.y"
     {
   yyval.triple=rasqal_new_triple(yyvsp[-3].literal, yyvsp[-2].literal, yyvsp[-1].literal);
 }
     break;
 
   case 19:
-#line 289 "./rdql_parser.y"
+#line 296 "./rdql_parser.y"
     {
   yyval.seq=NULL;
 }
     break;
 
   case 20:
-#line 293 "./rdql_parser.y"
+#line 300 "./rdql_parser.y"
     {
   yyval.seq=NULL;
 }
     break;
 
   case 21:
-#line 299 "./rdql_parser.y"
+#line 306 "./rdql_parser.y"
     {
-  rasqal_query_add_constraint(((rasqal_query*)rq), yyvsp[0].expr);
+  raptor_sequence_push(((rasqal_query*)rq)->constraints_sequence, yyvsp[0].expr);
   yyval.seq=NULL;
 }
     break;
 
   case 22:
-#line 304 "./rdql_parser.y"
+#line 311 "./rdql_parser.y"
     {
-  rasqal_query_add_constraint(((rasqal_query*)rq), yyvsp[0].expr);
+  raptor_sequence_push(((rasqal_query*)rq)->constraints_sequence, yyvsp[0].expr);
   yyval.seq=NULL;
 }
     break;
 
   case 23:
-#line 309 "./rdql_parser.y"
+#line 316 "./rdql_parser.y"
     {
-  rasqal_query_add_constraint(((rasqal_query*)rq), yyvsp[0].expr);
+  raptor_sequence_push(((rasqal_query*)rq)->constraints_sequence, yyvsp[0].expr);
   yyval.seq=NULL;
 }
     break;
 
   case 24:
-#line 318 "./rdql_parser.y"
+#line 325 "./rdql_parser.y"
     {
   yyval.seq=yyvsp[0].seq;
 }
     break;
 
   case 25:
-#line 322 "./rdql_parser.y"
+#line 329 "./rdql_parser.y"
     {
   yyval.seq=NULL;
 }
     break;
 
   case 26:
-#line 328 "./rdql_parser.y"
+#line 335 "./rdql_parser.y"
     {
   yyval.seq=((rasqal_query*)rq)->prefixes;
   raptor_sequence_shift(yyval.seq, rasqal_new_prefix(yyvsp[-4].name, yyvsp[-2].uri));
@@ -1425,7 +1432,7 @@ yyreduce:
     break;
 
   case 27:
-#line 333 "./rdql_parser.y"
+#line 340 "./rdql_parser.y"
     {
   yyval.seq=((rasqal_query*)rq)->prefixes;
   raptor_sequence_shift(yyval.seq, rasqal_new_prefix(yyvsp[-3].name, yyvsp[-1].uri));
@@ -1433,7 +1440,7 @@ yyreduce:
     break;
 
   case 28:
-#line 338 "./rdql_parser.y"
+#line 345 "./rdql_parser.y"
     {
   yyval.seq=((rasqal_query*)rq)->prefixes;
   raptor_sequence_push(yyval.seq, rasqal_new_prefix(yyvsp[-2].name, yyvsp[0].uri));
@@ -1441,21 +1448,21 @@ yyreduce:
     break;
 
   case 29:
-#line 346 "./rdql_parser.y"
+#line 353 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_OR, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 30:
-#line 350 "./rdql_parser.y"
+#line 357 "./rdql_parser.y"
     {
   yyval.expr=yyvsp[0].expr;
 }
     break;
 
   case 31:
-#line 356 "./rdql_parser.y"
+#line 363 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_AND, yyvsp[-2].expr, yyvsp[0].expr);
 ;
@@ -1463,196 +1470,196 @@ yyreduce:
     break;
 
   case 32:
-#line 361 "./rdql_parser.y"
+#line 368 "./rdql_parser.y"
     {
   yyval.expr=yyvsp[0].expr;
 }
     break;
 
   case 33:
-#line 367 "./rdql_parser.y"
+#line 374 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_STR_EQ, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 34:
-#line 371 "./rdql_parser.y"
+#line 378 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_STR_NEQ, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 35:
-#line 375 "./rdql_parser.y"
+#line 382 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_string_op_expression(RASQAL_EXPR_STR_MATCH, yyvsp[-2].expr, yyvsp[0].literal);
 }
     break;
 
   case 36:
-#line 379 "./rdql_parser.y"
+#line 386 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_string_op_expression(RASQAL_EXPR_STR_NMATCH, yyvsp[-2].expr, yyvsp[0].literal);
 }
     break;
 
   case 37:
-#line 383 "./rdql_parser.y"
+#line 390 "./rdql_parser.y"
     {
   yyval.expr=yyvsp[0].expr;
 }
     break;
 
   case 38:
-#line 389 "./rdql_parser.y"
+#line 396 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_EQ, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 39:
-#line 393 "./rdql_parser.y"
+#line 400 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_NEQ, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 40:
-#line 397 "./rdql_parser.y"
+#line 404 "./rdql_parser.y"
     {
   yyval.expr=yyvsp[0].expr;
 }
     break;
 
   case 41:
-#line 403 "./rdql_parser.y"
+#line 410 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_LT, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 42:
-#line 407 "./rdql_parser.y"
+#line 414 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_GT, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 43:
-#line 411 "./rdql_parser.y"
+#line 418 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_LE, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 44:
-#line 415 "./rdql_parser.y"
+#line 422 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_GE, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 45:
-#line 419 "./rdql_parser.y"
+#line 426 "./rdql_parser.y"
     {
   yyval.expr=yyvsp[0].expr;
 }
     break;
 
   case 46:
-#line 425 "./rdql_parser.y"
+#line 432 "./rdql_parser.y"
     {
   yyval.expr=yyvsp[0].expr;
 }
     break;
 
   case 47:
-#line 432 "./rdql_parser.y"
+#line 439 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_PLUS, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 48:
-#line 436 "./rdql_parser.y"
+#line 443 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_MINUS, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 49:
-#line 440 "./rdql_parser.y"
+#line 447 "./rdql_parser.y"
     {
   yyval.expr=yyvsp[0].expr;
 }
     break;
 
   case 50:
-#line 446 "./rdql_parser.y"
+#line 453 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_STAR, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 51:
-#line 450 "./rdql_parser.y"
+#line 457 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_SLASH, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 52:
-#line 454 "./rdql_parser.y"
+#line 461 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_2op_expression(RASQAL_EXPR_REM, yyvsp[-2].expr, yyvsp[0].expr);
 }
     break;
 
   case 53:
-#line 458 "./rdql_parser.y"
+#line 465 "./rdql_parser.y"
     {
   yyval.expr=yyvsp[0].expr;
 }
     break;
 
   case 54:
-#line 464 "./rdql_parser.y"
+#line 471 "./rdql_parser.y"
     {
   yyval.expr=yyvsp[0].expr;
 }
     break;
 
   case 55:
-#line 468 "./rdql_parser.y"
+#line 475 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_1op_expression(RASQAL_EXPR_UMINUS, yyvsp[0].expr);
 }
     break;
 
   case 56:
-#line 472 "./rdql_parser.y"
+#line 479 "./rdql_parser.y"
     {
   yyval.expr=yyvsp[0].expr;
 }
     break;
 
   case 57:
-#line 478 "./rdql_parser.y"
+#line 485 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_1op_expression(RASQAL_EXPR_TILDE, yyvsp[0].expr);
 }
     break;
 
   case 58:
-#line 482 "./rdql_parser.y"
+#line 489 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_1op_expression(RASQAL_EXPR_BANG, yyvsp[0].expr);
 }
     break;
 
   case 59:
-#line 486 "./rdql_parser.y"
+#line 493 "./rdql_parser.y"
     {
   rasqal_literal *l=rasqal_new_variable_literal(yyvsp[0].variable);
   yyval.expr=rasqal_new_literal_expression(l);
@@ -1660,119 +1667,119 @@ yyreduce:
     break;
 
   case 60:
-#line 491 "./rdql_parser.y"
+#line 498 "./rdql_parser.y"
     {
   yyval.expr=rasqal_new_literal_expression(yyvsp[0].literal);
 }
     break;
 
   case 61:
-#line 495 "./rdql_parser.y"
+#line 502 "./rdql_parser.y"
     {
   yyval.expr=yyvsp[-1].expr;
 }
     break;
 
   case 62:
-#line 501 "./rdql_parser.y"
+#line 508 "./rdql_parser.y"
     {
   yyval.literal=rasqal_new_variable_literal(yyvsp[0].variable);
 }
     break;
 
   case 63:
-#line 505 "./rdql_parser.y"
+#line 512 "./rdql_parser.y"
     {
   yyval.literal=rasqal_new_uri_literal(yyvsp[0].uri);
 }
     break;
 
   case 64:
-#line 509 "./rdql_parser.y"
+#line 516 "./rdql_parser.y"
     {
   yyval.literal=rasqal_new_simple_literal(RASQAL_LITERAL_QNAME, yyvsp[0].name);
 }
     break;
 
   case 65:
-#line 515 "./rdql_parser.y"
+#line 522 "./rdql_parser.y"
     {
   yyval.literal=rasqal_new_variable_literal(yyvsp[0].variable);
 }
     break;
 
   case 66:
-#line 519 "./rdql_parser.y"
+#line 526 "./rdql_parser.y"
     {
   yyval.literal=yyvsp[0].literal;
 }
     break;
 
   case 67:
-#line 525 "./rdql_parser.y"
+#line 532 "./rdql_parser.y"
     {
   yyval.variable=rasqal_new_variable((rasqal_query*)rq, yyvsp[0].name, NULL);
 }
     break;
 
   case 68:
-#line 531 "./rdql_parser.y"
+#line 538 "./rdql_parser.y"
     {
   yyval.literal=yyvsp[0].literal;
 }
     break;
 
   case 69:
-#line 537 "./rdql_parser.y"
+#line 544 "./rdql_parser.y"
     {
   yyval.literal=rasqal_new_uri_literal(yyvsp[0].uri);
 }
     break;
 
   case 70:
-#line 541 "./rdql_parser.y"
+#line 548 "./rdql_parser.y"
     {
   yyval.literal=yyvsp[0].literal;
 }
     break;
 
   case 71:
-#line 545 "./rdql_parser.y"
+#line 552 "./rdql_parser.y"
     {
   yyval.literal=yyvsp[0].literal;
 }
     break;
 
   case 72:
-#line 549 "./rdql_parser.y"
+#line 556 "./rdql_parser.y"
     {
   yyval.literal=yyvsp[0].literal;
 }
     break;
 
   case 73:
-#line 553 "./rdql_parser.y"
+#line 560 "./rdql_parser.y"
     {
   yyval.literal=yyvsp[0].literal;
 }
     break;
 
   case 74:
-#line 557 "./rdql_parser.y"
+#line 564 "./rdql_parser.y"
     {
   yyval.literal=yyvsp[0].literal;
 }
     break;
 
   case 75:
-#line 560 "./rdql_parser.y"
+#line 567 "./rdql_parser.y"
     {
   yyval.literal=rasqal_new_simple_literal(RASQAL_LITERAL_QNAME, yyvsp[0].name);
 }
     break;
 
   case 76:
-#line 567 "./rdql_parser.y"
+#line 574 "./rdql_parser.y"
     {
   yyval.seq=yyvsp[0].seq;
   raptor_sequence_shift(yyval.seq, yyvsp[-2].uri);
@@ -1780,7 +1787,7 @@ yyreduce:
     break;
 
   case 77:
-#line 572 "./rdql_parser.y"
+#line 579 "./rdql_parser.y"
     {
   yyval.seq=raptor_new_sequence((raptor_sequence_free_handler*)raptor_free_uri, (raptor_sequence_print_handler*)raptor_sequence_print_uri);
   raptor_sequence_push(yyval.seq, yyvsp[0].uri);
@@ -1791,7 +1798,7 @@ yyreduce:
     }
 
 /* Line 1000 of yacc.c.  */
-#line 1795 "rdql_parser.c"
+#line 1802 "rdql_parser.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -2016,7 +2023,7 @@ yyreturn:
 }
 
 
-#line 578 "./rdql_parser.y"
+#line 585 "./rdql_parser.y"
 
 
 
@@ -2082,6 +2089,9 @@ rasqal_rdql_query_engine_prepare(rasqal_query* rdf_query) {
   
   if(!rdf_query->query_string)
     return 1;
+
+  /* for RDQL only, before the graph pattern is made */
+  rdf_query->constraints_sequence=raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_expression_print);
   
   rc=rdql_parse(rdf_query, rdf_query->query_string);
   if(rc)
@@ -2090,14 +2100,30 @@ rasqal_rdql_query_engine_prepare(rasqal_query* rdf_query) {
   gp=rasqal_new_graph_pattern_from_triples(rdf_query,
                                            rdf_query->triples,
                                            0, raptor_sequence_size(rdf_query->triples)-1,
-                                           0);
-  raptor_sequence_push(rdf_query->graph_patterns, gp);
+                                           RASQAL_GRAPH_PATTERN_OPERATOR_BASIC);
+
+  rdf_query->query_graph_pattern=gp;
+
+  /* Now assign the constraints to the graph pattern */
+  while(raptor_sequence_size(rdf_query->constraints_sequence)) {
+    rasqal_expression* e=(rasqal_expression*)raptor_sequence_pop(rdf_query->constraints_sequence);
+    rasqal_graph_pattern_add_constraint(gp, e);
+  }
+  raptor_free_sequence(rdf_query->constraints_sequence);
+
+  /* Only now can we handle the prefixes and qnames */
+  if(rasqal_engine_declare_prefixes(rdf_query) ||
+     rasqal_engine_expand_triple_qnames(rdf_query) ||
+     rasqal_engine_expand_query_constraints_qnames(rdf_query))
+    return 1;
+
   return rasqal_engine_prepare(rdf_query);
 }
 
 
 static int
-rasqal_rdql_query_engine_execute(rasqal_query* rdf_query) 
+rasqal_rdql_query_engine_execute(rasqal_query* rdf_query, 
+                                 rasqal_query_results* results) 
 {
   /* rasqal_rdql_query_engine* rdql=(rasqal_rdql_query_engine*)rdf_query->context; */
   
@@ -2113,7 +2139,7 @@ rdql_parse(rasqal_query* rq, const unsigned char *string) {
   char *buf=NULL;
   size_t len;
   void *buffer;
-
+  
   if(!string || !*string)
     return yy_init_globals(NULL); /* 0 but a way to use yy_init_globals */
 
@@ -2161,12 +2187,6 @@ rdql_parse(rasqal_query* rq, const unsigned char *string) {
   if(rq->failed)
     return 1;
   
-  /* Only now can we handle the prefixes and qnames */
-  if(rasqal_engine_declare_prefixes(rq) ||
-     rasqal_engine_expand_triple_qnames(rq) ||
-     rasqal_engine_expand_constraints_qnames(rq))
-    return 1;
-
   return 0;
 }
 
@@ -2264,6 +2284,7 @@ main(int argc, char *argv[])
   FILE *fh;
   int rc;
   char *filename=NULL;
+  raptor_uri* base_uri=NULL;
   unsigned char *uri_string;
 
 #if RASQAL_DEBUG > 2
@@ -2294,14 +2315,16 @@ main(int argc, char *argv[])
   query=rasqal_new_query("rdql", NULL);
 
   uri_string=raptor_uri_filename_to_uri_string(filename);
-  query->base_uri=raptor_new_uri(uri_string);
-
-  rc=rdql_parse(query, (const unsigned char*)query_string);
+  base_uri=raptor_new_uri(uri_string);
+  
+  rc=rasqal_query_prepare(query, (const unsigned char*)query_string, base_uri);
 
   rasqal_query_print(query, stdout);
 
   rasqal_free_query(query);
 
+  raptor_free_uri(base_uri);
+
   raptor_free_memory(uri_string);
 
   rasqal_finish();
