@@ -2,7 +2,7 @@
  *
  * rdfproc.c - Redland RDF processor
  *
- * $Id: rdfproc.c,v 1.55 2005/02/03 22:58:54 cmdjb Exp $
+ * $Id: rdfproc.c,v 1.56 2005/05/18 11:13:46 cmdjb Exp $
  *
  * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -501,7 +501,7 @@ main(int argc, char *argv[])
     puts("  source means subject of triples matching (?,  NODE1, NODE2)");
     puts("  target means object of triples matching (NODE1, NODE2, ?)");
     puts("  arc means predicate of triples matching (NODE1, ?, NODE2)");
-    puts("\nReport bugs to <redland-dev@lists.librdf.org>.");
+    puts("\nReport bugs to http://bugs.librdf.org/");
     puts("Redland home page: http://librdf.org/");
     exit(0);
   }
